from mteb.abstasks import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_EVAL_SPLIT = "test"


class JQaRAReranking(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="JQaRAReranking",
        description=(
            "JQaRA: Japanese Question Answering with Retrieval Augmentation "
            " - 検索拡張(RAG)評価のための日本語 Q&A データセット. JQaRA is an information retrieval task "
            "for questions against 100 candidate data (including one or more correct answers)."
        ),
        reference="https://huggingface.co/datasets/hotchpotch/JQaRA",
        dataset={
            "path": "mteb/JQaRAReranking",
            "revision": "63254fa512e133e6300e35b6f9175c32c6cc8455",
        },
        type="Reranking",
        category="t2t",
        modalities=["text"],
        eval_splits=[_EVAL_SPLIT],
        eval_langs=["jpn-Jpan"],
        main_score="map_at_1000",
        date=("2020-01-01", "2024-12-31"),
        domains=["Encyclopaedic", "Non-fiction", "Written"],
        task_subtypes=["Question answering"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=["jpn-Jpan"],
        sample_creation="found",
        prompt="Given a Japanese question, rerank passages based on their relevance for answering the question",
        bibtex_citation=r"""
@misc{yuichi-tateno-2024-jqara,
  author = {Yuichi Tateno},
  title = {JQaRA: Japanese Question Answering with Retrieval Augmentation - 検索拡張(RAG)評価のための日本語Q&Aデータセット},
  url = {https://huggingface.co/datasets/hotchpotch/JQaRA},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class HateSpeechPortugueseClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="HateSpeechPortugueseClassification",
        description="HateSpeechPortugueseClassification is a dataset of Portuguese tweets categorized with their sentiment (2 classes).",
        reference="https://aclanthology.org/W19-3510",
        dataset={
            "path": "mteb/HateSpeechPortugueseClassification",
            "revision": "46a9ecdbad49534ea2bd771edde6423e15a2364d",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["por-Latn"],
        main_score="accuracy",
        date=("2017-03-08", "2017-03-09"),
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{fortuna-etal-2019-hierarchically,
  address = {Florence, Italy},
  author = {Fortuna, Paula  and
Rocha da Silva, Jo{\~a}o  and
Soler-Company, Juan  and
Wanner, Leo  and
Nunes, S{\'e}rgio},
  booktitle = {Proceedings of the Third Workshop on Abusive Language Online},
  doi = {10.18653/v1/W19-3510},
  editor = {Roberts, Sarah T.  and
Tetreault, Joel  and
Prabhakaran, Vinodkumar  and
Waseem, Zeerak},
  month = aug,
  pages = {94--104},
  publisher = {Association for Computational Linguistics},
  title = {A Hierarchically-Labeled {P}ortuguese Hate Speech Dataset},
  url = {https://aclanthology.org/W19-3510},
  year = {2019},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class SentiRuEval2016Classification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SentiRuEval2016",
        dataset={
            "path": "mteb/SentiRuEval2016",
            "revision": "8507eab0deef37f040a750afbcb4dba7a7de9c16",
        },
        description="Russian sentiment analysis evaluation SentiRuEval-2016 devoted to reputation monitoring of banks "
        "and telecom companies in Twitter. We describe the task, data, the procedure of data preparation, "
        "and participants’ results.",
        reference="https://github.com/mokoron/sentirueval",
        type="Classification",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["rus-Cyrl"],
        main_score="accuracy",
        date=("2015-01-01", "2016-01-01"),
        domains=[],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{loukachevitch2016sentirueval,
  author = {Loukachevitch, NV and Rubtsova, Yu V},
  booktitle = {Computational Linguistics and Intellectual Technologies},
  pages = {416--426},
  title = {SentiRuEval-2016: overcoming time gap and data sparsity in tweet sentiment analysis},
  year = {2016},
}
""",
        superseded_by="SentiRuEval2016.v2",
    )


class SentiRuEval2016ClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SentiRuEval2016.v2",
        dataset={
            "path": "mteb/senti_ru_eval2016",
            "revision": "bfa4cbec1753ffed29a8244a4ec208cc9e6c09a0",
        },
        description="Russian sentiment analysis evaluation SentiRuEval-2016 devoted to reputation monitoring of banks and telecom companies in Twitter. We describe the task, data, the procedure of data preparation, and participants’ results. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://github.com/mokoron/sentirueval",
        type="Classification",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["rus-Cyrl"],
        main_score="accuracy",
        date=("2015-01-01", "2016-01-01"),
        domains=[],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{loukachevitch2016sentirueval,
  author = {Loukachevitch, NV and Rubtsova, Yu V},
  booktitle = {Computational Linguistics and Intellectual Technologies},
  pages = {416--426},
  title = {SentiRuEval-2016: overcoming time gap and data sparsity in tweet sentiment analysis},
  year = {2016},
}
""",
        adapted_from=["SentiRuEval2016Classification"],
    )



from mteb.abstasks import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class SICKNLSTS(AbsTaskSTS):
    fast_loading = True
    metadata = TaskMetadata(
        name="SICK-NL-STS",
        dataset={
            "path": "clips/mteb-nl-sick-sts-pr",
            "revision": "7f88f003fc4e37ed8cd9ade84e390d871b032fef",
        },
        description="SICK-NL (read: signal), a dataset targeting Natural Language Inference in Dutch. SICK-NL is "
        "obtained by translating the SICK dataset of (Marelli et al., 2014) from English into Dutch.",
        reference="https://aclanthology.org/2021.eacl-main.126/",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="cosine_spearman",
        date=("2012-01-01", "2017-12-31"),
        domains=["News", "Social", "Web", "Spoken", "Written"],
        task_subtypes=[],
        license="mit",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@inproceedings{wijnholds2021sick,
  author = {Wijnholds, Gijs and Moortgat, Michael},
  booktitle = {Proceedings of the 16th Conference of the European Chapter of the Association for Computational Linguistics: Main Volume},
  pages = {1474--1479},
  title = {SICK-NL: A Dataset for Dutch Natural Language Inference},
  year = {2021},
}
""",
        prompt="Haal semantisch vergelijkbare tekst op",
    )

    min_score = 0
    max_score = 5



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class BBSARDNLRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="bBSARDNLRetrieval",
        description="Building on the Belgian Statutory Article Retrieval Dataset (BSARD) in French, we introduce the "
        "bilingual version of this dataset, bBSARD. The dataset contains parallel Belgian statutory "
        "articles in both French and Dutch, along with legal questions from BSARD and their Dutch "
        "translation.",
        reference="https://aclanthology.org/2025.regnlp-1.3.pdf",
        dataset={
            "path": "clips/mteb-nl-bbsard",
            "revision": "52027c212ba9765a3e9737c9cbf9a06ae83cbb93",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="ndcg_at_10",
        date=("2021-05-01", "2021-08-26"),
        domains=["Legal", "Written"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{lotfi2025bilingual,
  author = {Lotfi, Ehsan and Banar, Nikolay and Yuzbashyan, Nerses and Daelemans, Walter},
  journal = {COLING 2025},
  pages = {10},
  title = {Bilingual BSARD: Extending Statutory Article Retrieval to Dutch},
  year = {2025},
}
""",
        prompt={
            "query": "Gegeven een juridische vraag, haal documenten op die kunnen helpen bij het beantwoorden van de vraag"
        },
    )



from datasets import Audio, DatasetDict, load_dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_EVAL_LANGS = {
    "en": ["eng-Latn"],
    "fr": ["fra-Latn"],
    "de": ["deu-Latn"],
    "es": ["spa-Latn"],
}


def _load_jam_alt_data(
    splits,
    langs,
    dataset_args,
    query_column: str,
    corpus_column: str,
    qrels_column: str,
    **kwargs,
):
    corpus = {lang: dict.fromkeys(splits) for lang in langs}
    queries = {lang: dict.fromkeys(splits) for lang in langs}
    relevant_docs = {lang: dict.fromkeys(splits) for lang in langs}

    split = "test"
    ds = load_dataset(**dataset_args, split=split)

    # Cast audio column once if needed
    if "audio" in ds.column_names:
        ds = ds.cast_column("audio", Audio(decode=True))

    # Group indices by language
    lang_indices = {lang: [] for lang in langs}
    for idx, row in enumerate(ds):
        lang = row["song_language"]
        if lang in lang_indices:
            lang_indices[lang].append(idx)

    for lang, indices in lang_indices.items():
        if not indices:
            continue

        # Use select to avoid copying data
        lang_ds = ds.select(indices)

        # Generate IDs using map (avoids creating new dataset)
        def add_corpus_id(example, idx):
            example["id"] = f"corpus-{example['song_name']}{example['line_indices'][0]}"
            return example

        def add_query_id(example, idx):
            example["id"] = f"query-{example['song_name']}{example['line_indices'][0]}"
            return example

        # Create corpus
        corpus_ds = lang_ds.map(add_corpus_id, with_indices=True)
        corpus_ds = corpus_ds.select_columns(["id", corpus_column])
        corpus[lang][split] = corpus_ds

        # Create queries
        query_ds = lang_ds.map(add_query_id, with_indices=True)
        query_ds = query_ds.select_columns(["id", query_column])
        queries[lang][split] = query_ds

        # Build qrels efficiently
        qrels = {}
        qrel_groups = {}

        # Group by qrels_column
        for row in lang_ds:
            qrel_key = row[qrels_column]
            query_id = f"query-{row['song_name']}{row['line_indices'][0]}"
            corpus_id = f"corpus-{row['song_name']}{row['line_indices'][0]}"

            if qrel_key not in qrel_groups:
                qrel_groups[qrel_key] = {"queries": set(), "corpus": set()}

            qrel_groups[qrel_key]["queries"].add(query_id)
            qrel_groups[qrel_key]["corpus"].add(corpus_id)

        # Create cross-product within each group
        for group in qrel_groups.values():
            for query_id in group["queries"]:
                if query_id not in qrels:
                    qrels[query_id] = {}
                for corpus_id in group["corpus"]:
                    qrels[query_id][corpus_id] = 1

        relevant_docs[lang][split] = qrels

    corpus = DatasetDict({lang: DatasetDict(splits) for lang, splits in corpus.items()})
    queries = DatasetDict(
        {lang: DatasetDict(splits) for lang, splits in queries.items()}
    )
    relevant_docs = DatasetDict(relevant_docs)

    return corpus, queries, relevant_docs


class JamAltArtistA2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="JamAltArtistA2ARetrieval",
        description="Given audio clip of a song (query), retrieve all songs from the same artist in the Jam-Alt-Lines dataset",
        reference="https://huggingface.co/datasets/jamendolyrics/jam-alt-lines",
        dataset={
            "path": "mteb/jam-alt-lines",
            "revision": "e2d97afa7333eb489f1d451f76079d921be7a68f",
            "name": "pure",
        },
        type="Any2AnyRetrieval",
        category="a2a",
        modalities=["audio"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS,
        main_score="ndcg_at_10",
        date=("2023-01-01", "2024-12-31"),
        domains=["Music"],
        task_subtypes=["Music Genre Classification"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{cifka-2024-jam-alt,
  author = {Ond{\v{r}}ej C{\'{\i}}fka and
Hendrik Schreiber and
Luke Miner and
Fabian{-}Robert St{\"{o}}ter},
  booktitle = {Proceedings of the 25th International Society for
Music Information Retrieval Conference},
  doi = {10.5281/ZENODO.14877443},
  pages = {737--744},
  publisher = {ISMIR},
  title = {Lyrics Transcription for Humans: {A} Readability-Aware Benchmark},
  url = {https://doi.org/10.5281/zenodo.14877443},
  year = {2024},
}
""",
        prompt={
            "query": "Retrieve all songs created by the same artist as the following audio clip: {query}"
        },
    )
    skip_first_result = True

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = _load_jam_alt_data(
            splits=self.metadata.eval_splits,
            langs=self.hf_subsets,
            dataset_args=self.metadata.dataset,
            query_column="audio",
            corpus_column="audio",
            qrels_column="artist",
        )

        self.data_loaded = True


class JamAltLyricT2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="JamAltLyricT2ARetrieval",
        description="From textual lyrics (query), retrieve corresponding audio clips of songs from the Jam-Alt-Lines dataset",
        reference="https://huggingface.co/datasets/jamendolyrics/jam-alt-lines",
        dataset={
            "path": "mteb/jam-alt-lines",
            "revision": "e2d97afa7333eb489f1d451f76079d921be7a68f",
            "name": "pure",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS,
        main_score="ndcg_at_10",
        date=("2023-01-01", "2024-12-31"),
        domains=["Music"],
        task_subtypes=["Song Lyrics Retrieval"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{cifka-2024-jam-alt,
  author = {Ond{\v{r}}ej C{\'{\i}}fka and
Hendrik Schreiber and
Luke Miner and
Fabian{-}Robert St{\"{o}}ter},
  booktitle = {Proceedings of the 25th International Society for
Music Information Retrieval Conference},
  doi = {10.5281/ZENODO.14877443},
  pages = {737--744},
  publisher = {ISMIR},
  title = {Lyrics Transcription for Humans: {A} Readability-Aware Benchmark},
  url = {https://doi.org/10.5281/zenodo.14877443},
  year = {2024},
}
""",
        prompt={"query": "Retrieve audio clip for the lyrics: {query}"},
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = _load_jam_alt_data(
            splits=self.metadata.eval_splits,
            langs=self.hf_subsets,
            dataset_args=self.metadata.dataset,
            query_column="text",
            corpus_column="audio",
            qrels_column="text",
        )

        self.data_loaded = True


class JamAltLyricA2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="JamAltLyricA2TRetrieval",
        description="From audio clips of songs (query), retrieve corresponding textual lyric from the Jam-Alt-Lines dataset",
        reference="https://huggingface.co/datasets/jamendolyrics/jam-alt-lines",
        dataset={
            "path": "mteb/jam-alt-lines",
            "revision": "e2d97afa7333eb489f1d451f76079d921be7a68f",
            "name": "pure",
        },
        type="Any2AnyRetrieval",
        category="a2t",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS,
        main_score="ndcg_at_10",
        date=("2023-01-01", "2024-12-31"),
        domains=["Music"],
        task_subtypes=["Song Lyrics Retrieval"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{cifka-2024-jam-alt,
  author = {Ond{\v{r}}ej C{\'{\i}}fka and
Hendrik Schreiber and
Luke Miner and
Fabian{-}Robert St{\"{o}}ter},
  booktitle = {Proceedings of the 25th International Society for
Music Information Retrieval Conference},
  doi = {10.5281/ZENODO.14877443},
  pages = {737--744},
  publisher = {ISMIR},
  title = {Lyrics Transcription for Humans: {A} Readability-Aware Benchmark},
  url = {https://doi.org/10.5281/zenodo.14877443},
  year = {2024},
}
""",
        prompt={
            "query": "Retrieve textual lyric for the audio clips of songs: {query}"
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = _load_jam_alt_data(
            splits=self.metadata.eval_splits,
            langs=self.hf_subsets,
            dataset_args=self.metadata.dataset,
            query_column="audio",
            corpus_column="text",
            qrels_column="text",
        )

        self.data_loaded = True



from mteb.abstasks import AbsTaskClustering
from mteb.abstasks.task_metadata import TaskMetadata


class VehicleSoundClustering(AbsTaskClustering):
    metadata = TaskMetadata(
        name="VehicleSoundClustering",
        description="Clustering vehicle sounds recorded from smartphones (0 (car class), 1 (truck, bus and van class), 2 (motorcycle class))",
        reference="https://huggingface.co/datasets/DynamicSuperb/Vehicle_sounds_classification_dataset",
        dataset={
            "path": "mteb/Vehicle_sounds_classification_dataset",
            "revision": "5f905f9544d0c78b607639662c88c2ddd1216384",
        },
        type="AudioClustering",
        category="a2a",
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="v_measure",
        date=("2024-06-20", "2024-06-20"),
        domains=["Scene"],
        task_subtypes=["Vehicle Clustering"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{inproceedings,
  author = {Bazilinskyy, Pavlo and Aa, Arne and Schoustra, Michael and Spruit, John and Staats, Laurens and van der Vlist, Klaas Jan and de Winter, Joost},
  month = {05},
  pages = {},
  title = {An auditory dataset of passing vehicles recorded with a smartphone},
  year = {2018},
}
""",
    )

    label_column_name: str = "label"
    input_column_name: str = "audio"
    max_fraction_of_documents_to_embed = None



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import numpy as np
import torch

import mteb
from mteb._create_dataloaders import _corpus_to_dict
from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import SentenceTransformerEncoderWrapper
from mteb.types import PromptType

from .bge_models import bge_full_data

if TYPE_CHECKING:
    from collections.abc import Sequence

    from torch.utils.data import DataLoader

    from mteb.abstasks import (
        AbsTaskClassification,
        AbsTaskRetrieval,
        AbsTaskSummarization,
    )
    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput
logger = logging.getLogger(__name__)

CDE_CITATION = """@misc{morris2024contextualdocumentembeddings,
    title={Contextual Document Embeddings},
    author={John X. Morris and Alexander M. Rush},
    year={2024},
    eprint={2410.02525},
    archivePrefix={arXiv},
    primaryClass={cs.CL},
    url={https://arxiv.org/abs/2410.02525},
}"""


class CDEWrapper(SentenceTransformerEncoderWrapper):
    dataset_embeddings: torch.Tensor | None = None
    prev_embeddings_key: tuple[str, str] | None = None
    classification_task_types = (
        "Classification",
        "MultilabelClassification",
    )
    retrieval_task_types = (
        "Retrieval",
        "Reranking",
        "InstructionRetrieval",
        "InstructionReranking",
    )

    def __init__(
        self,
        model: str,
        revision: str | None = None,
        device: str | None = None,
        *args,
        **kwargs: Any,
    ) -> None:
        from transformers import AutoConfig

        super().__init__(model, revision=revision, device=device, *args, **kwargs)
        model_config = AutoConfig.from_pretrained(model, trust_remote_code=True)
        self.max_sentences = model_config.transductive_corpus_size

    def _create_embeddings_key(
        self,
        task_metadata: TaskMetadata,
        hf_subset: str,
    ) -> tuple[str, str]:
        """Create a key for the embeddings based on task metadata and subset.

        Args:
            task_metadata: Task metadata containing the task name.
            hf_subset: The subset of the task, e.g., "train", "test", etc.

        Returns:
            A tuple containing the task name and the subset.
        """
        return (
            task_metadata.name,
            hf_subset,
        )

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        prompt = self.get_prompt(task_metadata, prompt_type)
        if prompt:
            logger.info(
                f"Using prompt=`{prompt}` for task={task_metadata.name} prompt_type={prompt_type}"
            )
        else:
            logger.info(
                f"No model prompts found for task={task_metadata.name} prompt_type={prompt_type}"
            )
        sentences = [text for batch in inputs for text in batch["text"]]
        self._load_task_sample(
            sentences,
            task_metadata,
            prompt_type,
            hf_split=hf_split,
            hf_subset=hf_subset,
            **kwargs,
        )

        logger.info(f"Encoding {len(sentences)} sentences.")
        if self.dataset_embeddings is None:
            raise ValueError("Dataset embeddings are not loaded")

        embeddings = self.model.encode(
            sentences,
            prompt=prompt,
            dataset_embeddings=self.dataset_embeddings,
            **kwargs,
        )
        if isinstance(embeddings, torch.Tensor):
            # sometimes in kwargs can be return_tensors=True
            embeddings = embeddings.cpu().detach().float().numpy()

        self.prev_embeddings_key = self._create_embeddings_key(task_metadata, hf_subset)
        return embeddings

    def _load_task_sample(
        self,
        sentences: Sequence[str],
        task_metadata: TaskMetadata,
        prompt_type: PromptType | None,
        hf_split: str,
        hf_subset: str,
        **kwargs: Any,
    ) -> None:
        """Load a sample of the task dataset to create embeddings for the sentences.

        Args:
            sentences: List of sentences to encode.
            task_metadata: Task metadata containing the task name and type.
            prompt_type: The type of prompt to use (query or passage).
            hf_split: Split of the task, e.g., "train", "test", etc.
            hf_subset: Subset of the task, e.g., "train", "validation", etc.
            **kwargs: Encoding arguments to pass to the model.
        """
        if self.prev_embeddings_key == self._create_embeddings_key(
            task_metadata, hf_subset
        ):
            logger.info(
                f"Embeddings for {task_metadata.name} subset: {hf_subset} already loaded"
            )
            return
        prompt = "search_document: "
        logger.info(
            f"Loading dataset embeddings for task {task_metadata.name}, subset: {hf_subset} "
            f"Prompt: `{prompt}`"
        )
        if task_metadata.type in self.retrieval_task_types:
            task: AbsTaskRetrieval = mteb.get_task(task_metadata.name)
            task.load_data()
            task.convert_v1_dataset_format_to_v2()
            cur_ds = task.dataset[hf_subset][hf_split]["corpus"]
            sentences = cur_ds.map(_corpus_to_dict)["text"]
        elif task_metadata.type in self.classification_task_types:
            task: AbsTaskClassification = mteb.get_task(task_metadata.name)
            task.load_data()
            if hf_subset in task.dataset:
                cur_ds = task.dataset[hf_subset][task.train_split]
            else:
                cur_ds = task.dataset[task.train_split]
            sentences = cur_ds[task.input_column_name]
        elif task_metadata.type == "Summarization":
            task: AbsTaskSummarization = mteb.get_task(task_metadata.name)
            task.load_data()
            if hf_subset in task.dataset:
                cur_ds = task.dataset[hf_subset][hf_split]
            else:
                cur_ds = task.dataset[hf_split]
            sentences = cur_ds["text"]

        # We need to sample with replacement if the minicorpus needs to be bigger than the number of sentences
        is_replace = len(sentences) < self.max_sentences
        rng_state = np.random.default_rng(42)
        minicorpus = rng_state.choice(
            sentences, size=self.max_sentences, replace=is_replace
        )
        self.dataset_embeddings = self.model.encode(
            minicorpus,
            prompt=prompt,
            convert_to_tensor=True,
            **kwargs,
        )


cde_model_prompts = {
    # same as for nomic models
    "Classification": "classification: ",
    "MultilabelClassification": "classification: ",
    "Clustering": "clustering: ",
    "PairClassification": "classification: ",
    "Reranking": "classification: ",
    "STS": "classification: ",
    "Summarization": "classification: ",
    PromptType.query.value: "search_query: ",
    PromptType.document.value: "search_document: ",
}

cde_small_v1 = ModelMeta(
    loader=CDEWrapper,
    loader_kwargs=dict(
        model_prompts=cde_model_prompts,
        trust_remote_code=True,
    ),
    name="jxm/cde-small-v1",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="e151df18af0d7f1d1c37b074fee58406ececf19f",
    release_date="2024-09-24",
    n_parameters=int(281 * 1e6),
    n_embedding_parameters=None,
    memory_usage_mb=1072,  # Though the second-stage model is only 140M
    max_tokens=512,
    embed_dim=768,
    license="mit",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/jxm/cde-small-v1",
    use_instructions=True,
    adapted_from="nomic-ai/nomic-bert-2048",
    superseded_by="jxm/cde-small-v2",
    training_datasets=bge_full_data,
    public_training_code="https://github.com/jxmorris12/cde",
    public_training_data="https://huggingface.co/datasets/cfli/bge-full-data",
    citation=CDE_CITATION,
)

cde_small_v2 = ModelMeta(
    loader=CDEWrapper,
    loader_kwargs=dict(
        model_prompts=cde_model_prompts,
        trust_remote_code=True,
    ),
    name="jxm/cde-small-v2",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="4e1d021a6c3fd7ce8aa0a7204057eee5ae61d390",
    release_date="2025-01-13",
    n_parameters=int(306 * 1e6),
    n_embedding_parameters=None,
    memory_usage_mb=1166,  # Though the second-stage model is only 140M
    max_tokens=512,
    embed_dim=768,
    license="mit",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/jxm/cde-small-v1",
    use_instructions=True,
    adapted_from="answerdotai/ModernBERT-base",
    superseded_by="jxm/cde-small-v2",
    training_datasets=bge_full_data,
    public_training_code="https://github.com/jxmorris12/cde",
    public_training_data="https://huggingface.co/datasets/cfli/bge-full-data",
    citation=CDE_CITATION,
)



from hashlib import sha256

import datasets

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class GreekCivicsQA(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="GreekCivicsQA",
        description="This dataset was provided by AlloProf, an organisation in Quebec, Canada offering resources and a help forum curated by a large number of teachers to students on all subjects taught from in primary and secondary school",
        reference="https://huggingface.co/datasets/antoinelb7/alloprof",
        dataset={
            "path": "ilsp/greek_civics_qa",
            "revision": "a04523a3c83153be07a8945bb1fb351cbbcef90b",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["default"],
        eval_langs=["ell-Grek"],
        main_score="ndcg_at_10",
        date=("2023-01-01", "2024-04-01"),
        domains=["Academic", "Written"],
        task_subtypes=["Question answering"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
    )

    def load_data(self, num_proc: int | None = None, **kwargs) -> None:
        if self.data_loaded:
            return
        # fetch both subsets of the dataset
        eval_split = self.metadata.eval_splits[0]
        data_raw = datasets.load_dataset(**self.metadata.dataset)[eval_split]

        queries = {eval_split: {}}
        corpus = {eval_split: {}}
        relevant_docs = {eval_split: {}}

        question_ids = {
            question: str(id)
            for id, question in zip(data_raw["id"], data_raw["question"])
        }

        context_ids = {
            answer: sha256(answer.encode("utf-8")).hexdigest()
            for answer in set(data_raw["answer"])
        }

        for row in data_raw:
            question = row["question"]
            context = row["answer"]
            query_id = question_ids[question]
            queries[eval_split][query_id] = question

            doc_id = context_ids[context]
            corpus[eval_split][doc_id] = {"text": context}
            if query_id not in relevant_docs[eval_split]:
                relevant_docs[eval_split][query_id] = {}
            relevant_docs[eval_split][query_id][doc_id] = 1

        self.corpus = datasets.DatasetDict(corpus)
        self.queries = datasets.DatasetDict(queries)
        self.relevant_docs = datasets.DatasetDict(relevant_docs)

        self.data_loaded = True



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class ChemNQRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="ChemNQRetrieval",
        dataset={
            "path": "BASF-AI/ChemNQRetrieval",
            "revision": "5d958fb6b31055495347724d46431ba41309b03a",
        },
        description="ChemTEB evaluates the performance of text embedding models on chemical domain data.",
        reference="https://arxiv.org/abs/2412.00532",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2024-06-01", "2024-11-30"),
        domains=["Chemistry"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{47761,
  author = {Tom Kwiatkowski and Jennimaria Palomaki and Olivia Redfield and Michael Collins and Ankur Parikh
and Chris Alberti and Danielle Epstein and Illia Polosukhin and Matthew Kelcey and Jacob Devlin and Kenton Lee
and Kristina N. Toutanova and Llion Jones and Ming-Wei Chang and Andrew Dai and Jakob Uszkoreit and Quoc Le
and Slav Petrov},
  journal = {Transactions of the Association of Computational Linguistics},
  title = {Natural Questions: a Benchmark for Question Answering Research},
  year = {2019},
}

@article{kasmaee2024chemteb,
  author = {Kasmaee, Ali Shiraee and Khodadad, Mohammad and Saloot, Mohammad Arshi and Sherck, Nick and Dokas, Stephen and Mahyar, Hamidreza and Samiee, Soheila},
  journal = {arXiv preprint arXiv:2412.00532},
  title = {ChemTEB: Chemical Text Embedding Benchmark, an Overview of Embedding Models Performance \& Efficiency on a Specific Domain},
  year = {2024},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class UrbanSound8kClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="UrbanSound8k",
        description="Environmental Sound Classification Dataset.",
        reference="https://huggingface.co/datasets/danavery/urbansound8K",
        dataset={
            "path": "mteb/urbansound8K",
            "revision": "5b3867ddd7583a24871acdf6eb5494696ddd4cbc",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["train"],
        eval_langs=["zxx-Zxxx"],
        main_score="accuracy",
        date=("2014-11-01", "2014-11-03"),
        domains=["AudioScene"],
        task_subtypes=["Environment Sound Classification"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Salamon:UrbanSound:ACMMM:14,
  author = {Salamon, Justin and Jacoby, Christopher and Bello, Juan Pablo},
  booktitle = {Proceedings of the 22nd ACM international conference on Multimedia},
  organization = {ACM},
  pages = {1041--1044},
  title = {A Dataset and Taxonomy for Urban Sound Research},
  year = {2014},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "classID"
    is_cross_validation: bool = True


def dataset_transform(self):
    self.dataset = self.stratified_subsampling(
        self.dataset, seed=self.seed, splits=["train"], label=self.label_column_name
    )



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class JSICK(AbsTaskSTS):
    metadata = TaskMetadata(
        name="JSICK",
        dataset={
            "path": "mteb/JSICK",
            "revision": "729cfe4a16d3c2b61c6aa9f9f6c8a96bb5512868",
        },
        description="JSICK is the Japanese NLI and STS dataset by manually translating the English dataset SICK (Marelli et al., 2014) into Japanese.",
        reference="https://github.com/sbintuitions/JMTEB",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["jpn-Jpan"],
        main_score="cosine_spearman",
        date=("2000-01-01", "2012-12-31"),  # best guess
        domains=["Web", "Written"],
        task_subtypes=[],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{yanaka2022compositional,
  author = {Yanaka, Hitomi and Mineshima, Koji},
  journal = {Transactions of the Association for Computational Linguistics},
  pages = {1266--1284},
  publisher = {MIT Press One Broadway, 12th Floor, Cambridge, Massachusetts 02142, USA~…},
  title = {Compositional Evaluation on Japanese Textual Entailment and Similarity},
  volume = {10},
  year = {2022},
}
""",
    )

    min_score = 1
    max_score = 5



from mteb.abstasks.pair_classification import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class DisCoTexPairClassification(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="DisCoTexPairClassification",
        description="The DisCoTEX dataset aims at assessing discourse coherence in Italian texts. This dataset focuses on Italian real-world texts and provides resources to model coherence in natural language.",
        reference="https://github.com/davidecolla/DisCoTex",
        dataset={
            "path": "MattiaSangermano/DisCoTex-last-sentence",
            "revision": "ab9ea43f8e54c8b24b12cd1b77d6eb462385a30b",
        },
        type="PairClassification",
        category="t2t",
        modalities=["text"],
        date=("2023-01-01", "2023-12-31"),
        eval_splits=["test"],
        eval_langs=["ita-Latn"],
        main_score="max_ap",
        domains=["Social", "Written"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{brunato2023discotex,
  author = {Brunato, Dominique and Colla, Davide and Dell'Orletta, Felice and Dini, Irene and Radicioni, Daniele Paolo and Ravelli, Andrea Amelio and others},
  booktitle = {CEUR WORKSHOP PROCEEDINGS},
  organization = {CEUR},
  pages = {1--8},
  title = {DisCoTex at EVALITA 2023: overview of the assessing discourse coherence in Italian texts task},
  volume = {3473},
  year = {2023},
}
""",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.dataset.remove_columns(["id", "source"])
        self.dataset = self.dataset.map(
            lambda x: {
                "prompt": [x["prompt"]],
                "target": [x["target"]],
                "class": [x["class"]],
            },
            batched=True,
            batch_size=len(self.dataset["train"]),
        )
        self.dataset = self.dataset.rename_column("prompt", "sentence1")
        self.dataset = self.dataset.rename_column("target", "sentence2")
        self.dataset = self.dataset.rename_column("class", "labels")



from collections import defaultdict

import datasets
from datasets import Audio, DatasetDict
from tqdm.auto import tqdm

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_EVAL_LANGS = {
    "ar_eg": ["arz-Arab"],  # Egyptian Arabic
    "ar_x_gulf": ["acm-Arab"],  # Gulf Arabic
    "ar_x_levant": ["apc-Arab"],  # Levantine Arabic
    "ar_x_maghrebi": ["arq-Arab"],  # Maghrebi Arabic
    "bn_bd": ["ben-Beng"],  # Bengali (Bangladesh)
    "bn_in": ["ben-Beng"],  # Bengali (India)
    "en_au": ["eng-Latn"],  # English (Australia)
    "en_gb": ["eng-Latn"],  # English (GB)
    "en_in": ["eng-Latn"],  # English (India)
    "en_ph": ["eng-Latn"],  # English (Philippines)
    "en_us": ["eng-Latn"],  # English (US)
    "fi_fi": ["fin-Latn"],  # Finnish
    "gu_in": ["guj-Gujr"],  # Gujarati (India)
    "hi_in": ["hin-Deva"],  # Hindi (India)
    "id_id": ["ind-Latn"],  # Indonesian
    "ja_jp": ["jpn-Jpan"],  # Japanese
    "kn_in": ["kan-Knda"],  # Kannada (India)
    "ko_kr": ["kor-Hang"],  # Korean
    "ml_in": ["mal-Mlym"],  # Malayalam (India)
    "mr_in": ["mar-Deva"],  # Marathi (India)
    "ru_ru": ["rus-Cyrl"],  # Russian
    "sw": ["swa-Latn"],  # Swahili
    "ta_in": ["tam-Taml"],  # Tamil (India)
    "te_in": ["tel-Telu"],  # Telugu (India)
    "ur_in": ["urd-Arab"],  # Urdu (India)
    "ur_pk": ["urd-Arab"],  # Urdu (Pakistan)
}


class GoogleSVQA2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="GoogleSVQA2TRetrieval",
        description="Multilingual audio-to-text retrieval using the Simple Voice Questions (SVQ) dataset. Given an audio query, retrieve the corresponding text transcription.",
        reference="https://huggingface.co/datasets/google/svq",
        dataset={
            "path": "mteb/svq",
            "revision": "dd5a15e925767554c71b867f811b7eab53c770a4",
            "name": "audio",
        },
        type="Any2AnyRetrieval",
        category="a2t",
        modalities=["audio", "text"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS,
        main_score="hit_rate_at_5",
        date=("2024-01-01", "2024-12-31"),
        domains=["Spoken"],
        task_subtypes=["Speech Transcription Retrieval"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{heigold2025massive,
  author = {Georg Heigold and Ehsan Variani and Tom Bagby and Cyril Allauzen and Ji Ma and Shankar Kumar and Michael Riley},
  booktitle = {The Thirty-ninth Annual Conference on Neural Information Processing Systems Datasets and Benchmarks Track},
  title = {Massive Sound Embedding Benchmark ({MSEB})},
  url = {https://openreview.net/forum?id=X0juYgFVng},
  year = {2025},
}
""",
    )

    def load_data(self, **kwargs):
        if getattr(self, "data_loaded", False):
            return
        self.corpus = defaultdict(DatasetDict)
        self.queries = defaultdict(DatasetDict)
        self.relevant_docs = defaultdict(DatasetDict)
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self, id_col="utt_id", text_col="text", audio_col="waveform"):
        """A2T: Query = audio, Corpus = text."""
        for split in self.metadata.eval_splits:
            full_dataset = datasets.load_dataset(
                self.metadata.dataset["path"],
                self.metadata.dataset.get("name", "audio"),
                split=split,
                revision=self.metadata.dataset.get("revision"),
            )

            # Cast once before filtering
            full_dataset = full_dataset.cast_column(audio_col, Audio(decode=True))

            # Group indices by locale in a single pass
            locale_indices = {locale: [] for locale in self.metadata.eval_langs}

            for idx, row in enumerate(tqdm(full_dataset, desc="Filtering")):
                locale = row["locale"]
                if locale in locale_indices:
                    locale_indices[locale].append(idx)

            # Use select with indices to avoid recreating dataset
            for locale, indices in tqdm(
                locale_indices.items(), total=len(locale_indices)
            ):
                if not indices:
                    continue

                lang_dataset = full_dataset.select(indices)

                queries_ds = (
                    lang_dataset.select_columns([id_col, audio_col])
                    .rename_column(id_col, "id")
                    .rename_column(audio_col, "audio")
                )

                corpus_ds = lang_dataset.select_columns(
                    [id_col, text_col]
                ).rename_column(id_col, "id")

                relevant_docs_ = {
                    str(row[id_col]): {str(row[id_col]): 1} for row in lang_dataset
                }

                self.corpus[locale][split] = corpus_ds
                self.queries[locale][split] = queries_ds
                self.relevant_docs[locale][split] = relevant_docs_


class GoogleSVQT2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="GoogleSVQT2ARetrieval",
        description="Multilingual text-to-audio retrieval using the Simple Voice Questions (SVQ) dataset. Given a text query, retrieve the corresponding audio recording.",
        reference="https://huggingface.co/datasets/google/svq",
        dataset={
            "path": "mteb/svq",
            "revision": "dd5a15e925767554c71b867f811b7eab53c770a4",
            "name": "audio",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS,
        main_score="hit_rate_at_5",
        date=("2024-01-01", "2024-12-31"),
        domains=["Spoken"],
        task_subtypes=["Speech Retrieval"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{heigold2025massive,
  author = {Georg Heigold and Ehsan Variani and Tom Bagby and Cyril Allauzen and Ji Ma and Shankar Kumar and Michael Riley},
  booktitle = {The Thirty-ninth Annual Conference on Neural Information Processing Systems Datasets and Benchmarks Track},
  title = {Massive Sound Embedding Benchmark ({MSEB})},
  url = {https://openreview.net/forum?id=X0juYgFVng},
  year = {2025},
}
""",
    )

    def load_data(self, **kwargs):
        if getattr(self, "data_loaded", False):
            return
        self.corpus = defaultdict(DatasetDict)
        self.queries = defaultdict(DatasetDict)
        self.relevant_docs = defaultdict(DatasetDict)
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self, id_col="utt_id", text_col="text", audio_col="waveform"):
        """T2A: Query = text, Corpus = audio."""
        for split in self.metadata.eval_splits:
            full_dataset = datasets.load_dataset(
                self.metadata.dataset["path"],
                self.metadata.dataset.get("name", "audio"),
                split=split,
                revision=self.metadata.dataset.get("revision"),
            )

            full_dataset = full_dataset.cast_column(audio_col, Audio(decode=True))

            locale_indices = {locale: [] for locale in self.metadata.eval_langs}

            for idx, row in enumerate(tqdm(full_dataset, desc="Filtering")):
                locale = row["locale"]
                if locale in locale_indices:
                    locale_indices[locale].append(idx)

            # Use select with indices to avoid recreating dataset
            for locale, indices in tqdm(
                locale_indices.items(), total=len(locale_indices)
            ):
                if not indices:
                    continue

                lang_dataset = full_dataset.select(indices)

                queries_ds = lang_dataset.select_columns(
                    [id_col, text_col]
                ).rename_column(id_col, "id")

                corpus_ds = (
                    lang_dataset.select_columns([id_col, audio_col])
                    .rename_column(id_col, "id")
                    .rename_column(audio_col, "audio")
                )

                relevant_docs_ = {
                    str(row[id_col]): {str(row[id_col]): 1} for row in lang_dataset
                }

                self.corpus[locale][split] = corpus_ds
                self.queries[locale][split] = queries_ds
                self.relevant_docs[locale][split] = relevant_docs_



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class UrbanSound8KA2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="UrbanSound8KA2TRetrieval",
        description="UrbanSound8K: Audio-to-text retrieval of urban sound events.",
        reference="https://huggingface.co/datasets/CLAPv2/Urbansound8K",
        dataset={
            "path": "mteb/Urbansound8K_a2t",
            "revision": "b8e64ca746c4798fd35cf85eb9a01500129b1976",
        },
        type="Any2AnyRetrieval",
        category="a2t",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="hit_rate_at_5",
        date=("2014-11-01", "2014-11-03"),
        domains=["AudioScene"],
        task_subtypes=["Environment Sound Retrieval"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Salamon:UrbanSound:ACMMM:14,
  author = {Salamon, Justin and Jacoby, Christopher and Bello, Juan Pablo},
  booktitle = {Proceedings of the 22nd ACM international conference on Multimedia},
  organization = {ACM},
  pages = {1041--1044},
  title = {A Dataset and Taxonomy for Urban Sound Research},
  year = {2014},
}
""",
    )


class UrbanSound8KT2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="UrbanSound8KT2ARetrieval",
        description="UrbanSound8K: Text-to-audio retrieval of urban sound events.",
        reference="https://huggingface.co/datasets/CLAPv2/Urbansound8K",
        dataset={
            "path": "mteb/Urbansound8K_t2a",
            "revision": "d50fd1daded60cf136b82d845b9bda65b8b2376e",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="hit_rate_at_5",
        date=("2014-11-01", "2014-11-03"),
        domains=["AudioScene"],
        task_subtypes=["Environment Sound Retrieval"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Salamon:UrbanSound:ACMMM:14,
  author = {Salamon, Justin and Jacoby, Christopher and Bello, Juan Pablo},
  booktitle = {Proceedings of the 22nd ACM international conference on Multimedia},
  organization = {ACM},
  pages = {1041--1044},
  title = {A Dataset and Taxonomy for Urban Sound Research},
  year = {2014},
}
""",
    )



from __future__ import annotations

from enum import Enum
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing_extensions import Self


class HelpfulStrEnum(str, Enum):
    """StrEnum that provides a method to create an instance from a string value, which allows for more user-friendly error messages."""

    @classmethod
    def from_str(cls, value: str) -> Self:
        try:
            return cls(value)
        except ValueError:
            raise ValueError(
                f"'{value}' is not a valid {cls.__name__} value, must be one of {[e.value for e in cls]}"
            )



from .humesib200_clustering_s2s import HUMESIB200ClusteringS2S
from .indic_reviews_clustering_p2p import IndicReviewsClusteringP2P
from .masakha_news_clustering_p2p import MasakhaNEWSClusteringP2P
from .masakha_news_clustering_s2s import MasakhaNEWSClusteringS2S
from .mlsum_clustering_p2p import MLSUMClusteringP2P, MLSUMClusteringP2PFast
from .mlsum_clustering_s2s import MLSUMClusteringS2S, MLSUMClusteringS2SFast
from .sib200_clustering_s2s import SIB200ClusteringFast
from .vox_populi_gender_clustering import VoxPopuliGenderClustering
from .wiki_clustering_p2p import WikiClusteringFastP2P, WikiClusteringP2P

__all__ = [
    "HUMESIB200ClusteringS2S",
    "IndicReviewsClusteringP2P",
    "MLSUMClusteringP2P",
    "MLSUMClusteringP2PFast",
    "MLSUMClusteringS2S",
    "MLSUMClusteringS2SFast",
    "MasakhaNEWSClusteringP2P",
    "MasakhaNEWSClusteringS2S",
    "SIB200ClusteringFast",
    "VoxPopuliGenderClustering",
    "WikiClusteringFastP2P",
    "WikiClusteringP2P",
]



from .iso_mappings import (
    ISO_TO_LANGUAGE,
    ISO_TO_SCRIPT,
    path_to_lang_codes,
    path_to_lang_scripts,
)
from .programming_languages import PROGRAMMING_LANGS


def check_language_code(code: str) -> None:
    """This method checks that the language code (e.g. "eng-Latn") is valid.

    Args:
        code: The language code to check.
    """
    lang = None
    script = None
    if "-" in code:
        lang, script = code.split("-")
    elif code[0].isupper():
        script = code
    else:
        lang = code

    if script == "Code":
        if lang in PROGRAMMING_LANGS:
            return  # override for code
        else:
            raise ValueError(
                f"Programming language {lang} is not a valid programming language."
            )
    if lang is not None and lang not in ISO_TO_LANGUAGE:
        raise ValueError(
            f"Invalid language code: {lang}, you can find valid ISO 639-3 codes in {path_to_lang_codes}"
        )
    if script is not None and script not in ISO_TO_SCRIPT:
        raise ValueError(
            f"Invalid script code: {script}, you can find valid ISO 15924 codes in {path_to_lang_scripts}"
        )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class JavaneseIMDBClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="JavaneseIMDBClassification",
        description="Large Movie Review Dataset translated to Javanese. This is a dataset for binary sentiment classification containing substantially more data than previous benchmark datasets.",
        reference="https://github.com/w11wo/nlp-datasets#javanese-imdb",
        dataset={
            "path": "mteb/JavaneseIMDBClassification",
            "revision": "6bf102a9551cc100314a97a470f07b6e43f3f346",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2021-06-24", "2021-06-24"),
        eval_splits=["test"],
        eval_langs=["jav-Latn"],
        main_score="accuracy",
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="mit",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{wongso2021causal,
  author = {Wongso, Wilson and Setiawan, David Samuel and Suhartono, Derwin},
  booktitle = {2021 International Conference on Advanced Computer Science and Information Systems (ICACSIS)},
  organization = {IEEE},
  pages = {1--7},
  title = {Causal and Masked Language Modeling of Javanese Language using Transformer-based Architectures},
  year = {2021},
}
""",
        superseded_by="JavaneseIMDBClassification.v2",
    )


class JavaneseIMDBClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="JavaneseIMDBClassification.v2",
        description="Large Movie Review Dataset translated to Javanese. This is a dataset for binary sentiment classification containing substantially more data than previous benchmark datasets. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://github.com/w11wo/nlp-datasets#javanese-imdb",
        dataset={
            "path": "mteb/javanese_imdb",
            "revision": "47aadc77049fa4e7b9001c69a255555814d026d9",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2021-06-24", "2021-06-24"),
        eval_splits=["test"],
        eval_langs=["jav-Latn"],
        main_score="accuracy",
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="mit",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{wongso2021causal,
  author = {Wongso, Wilson and Setiawan, David Samuel and Suhartono, Derwin},
  booktitle = {2021 International Conference on Advanced Computer Science and Information Systems (ICACSIS)},
  organization = {IEEE},
  pages = {1--7},
  title = {Causal and Masked Language Modeling of Javanese Language using Transformer-based Architectures},
  year = {2021},
}
""",
        adapted_from=["JavaneseIMDBClassification"],
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )



from datasets import load_dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class HunSum2AbstractiveRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="HunSum2AbstractiveRetrieval",
        dataset={
            "path": "SZTAKI-HLT/HunSum-2-abstractive",
            "revision": "24e1445c8180d937f0a16f8ae8a62e77cc952e56",
        },
        description=(
            "HunSum-2-abstractive is a Hungarian dataset containing news articles along with lead, titles and metadata."
        ),
        reference="https://arxiv.org/abs/2404.03555",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["hun-Latn"],
        main_score="ndcg_at_1",
        date=(
            "1848-12-15",
            "2024-03-19",
        ),
        domains=["News", "Written"],
        task_subtypes=["Article retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{barta2024news,
  archiveprefix = {arXiv},
  author = {Botond Barta and Dorina Lakatos and Attila Nagy and Milán Konor Nyist and Judit Ács},
  eprint = {2404.03555},
  primaryclass = {cs.CL},
  title = {From News to Summaries: Building a Hungarian Corpus for Extractive and Abstractive Summarization},
  year = {2024},
}
""",
    )

    def load_data(self, num_proc: int | None = None, **kwargs) -> None:
        if self.data_loaded:
            return
        self.corpus, self.queries, self.relevant_docs = {}, {}, {}
        ds = load_dataset(**self.metadata.dataset, split=self.metadata.eval_splits)
        ds = dict(zip(self.metadata.eval_splits, ds))
        for split_name, split in ds.items():
            self.corpus[split_name] = {}
            self.queries[split_name] = {}
            self.relevant_docs[split_name] = {}
            for record in split:
                self.corpus[split_name]["d" + record["uuid"]] = {
                    "title": record["title"],
                    "text": record["article"],
                }
                self.queries[split_name]["q" + record["uuid"]] = record["lead"]
                self.relevant_docs[split_name]["q" + record["uuid"]] = {
                    "d" + record["uuid"]: 1
                }
        self.data_loaded = True



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SOPI2IRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SOPI2IRetrieval",
        description="Retrieve product photos of 22634 online products.",
        reference="https://paperswithcode.com/dataset/stanford-online-products",
        dataset={
            "path": "mteb/stanford-online-products",
            "revision": "9556adace66a20ac7ef034b6b9dd854c0ed9779f",
        },
        type="Any2AnyRetrieval",
        category="i2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_1",
        date=("2019-07-17", "2019-07-17"),
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{oh2016deep,
  author = {Oh Song, Hyun and Xiang, Yu and Jegelka, Stefanie and Savarese, Silvio},
  booktitle = {Proceedings of the IEEE conference on computer vision and pattern recognition},
  pages = {4004--4012},
  title = {Deep metric learning via lifted structured feature embedding},
  year = {2016},
}
""",
    )
    skip_first_result = True



from __future__ import annotations

import difflib
import logging
import warnings
from typing import TYPE_CHECKING, Any

from mteb.models import (
    ModelMeta,
)
from mteb.models.model_implementations import MODEL_REGISTRY

if TYPE_CHECKING:
    from collections.abc import Iterable, Mapping

    from mteb.abstasks import AbsTask
    from mteb.models import (
        MTEBModels,
    )

logger = logging.getLogger(__name__)


def get_model_metas(
    model_names: Iterable[str] | None = None,
    languages: Iterable[str] | None = None,
    open_weights: bool | None = None,
    frameworks: Iterable[str] | None = None,
    n_parameters_range: tuple[int | None, int | None] = (None, None),
    use_instructions: bool | None = None,
    zero_shot_on: list[AbsTask] | None = None,
    model_types: Iterable[str] | None = None,
) -> list[ModelMeta]:
    """Load all models' metadata that fit the specified criteria.

    Args:
        model_names: A list of model names to filter by. If None, all models are included.
        languages: A list of languages to filter by. If None, all languages are included.
        open_weights: Whether to filter by models with open weights. If None this filter is ignored.
        frameworks: A list of frameworks to filter by. If None, all frameworks are included.
        n_parameters_range: A tuple of lower and upper bounds of the number of parameters to filter by.
            If (None, None), this filter is ignored.
        use_instructions: Whether to filter by models that use instructions. If None, all models are included.
        zero_shot_on: A list of tasks on which the model is zero-shot. If None this filter is ignored.
        model_types: A list of model types to filter by. If None, all model types are included.

    Returns:
        A list of model metadata objects that fit the specified criteria.
    """
    res = []
    model_names = set(model_names) if model_names is not None else None
    languages = set(languages) if languages is not None else None
    frameworks = set(frameworks) if frameworks is not None else None
    model_types_set = set(model_types) if model_types is not None else None
    for model_meta in MODEL_REGISTRY.values():
        if (model_names is not None) and (model_meta.name not in model_names):
            continue
        if languages is not None:
            if (model_meta.languages is None) or not (
                languages <= set(model_meta.languages)
            ):
                continue
        if (open_weights is not None) and (model_meta.open_weights != open_weights):
            continue
        if (frameworks is not None) and not (frameworks <= set(model_meta.framework)):
            continue
        if (use_instructions is not None) and (
            model_meta.use_instructions != use_instructions
        ):
            continue
        if model_types_set is not None and not model_types_set.intersection(
            model_meta.model_type
        ):
            continue

        lower, upper = n_parameters_range
        n_parameters = model_meta.n_parameters

        if upper is not None:
            if (n_parameters is None) or (n_parameters > upper):
                continue
            if lower is not None and n_parameters < lower:
                continue

        if zero_shot_on is not None:
            if not model_meta.is_zero_shot_on(zero_shot_on):
                continue
        res.append(model_meta)
    return res


def get_model(
    model_name: str,
    revision: str | None = None,
    device: str | None = None,
    **kwargs: Any,
) -> MTEBModels:
    """A function to fetch and load model object by name.

    !!! note
        This function loads the model into memory. If you only want to fetch the metadata, use [`get_model_meta`](#mteb.get_model_meta) instead.

    Args:
        model_name: Name of the model to fetch
        revision: Revision of the model to fetch
        device: Device used to load the model
        **kwargs: Additional keyword arguments to pass to the model loader

    Returns:
        A model object
    """
    meta = get_model_meta(model_name, revision).model_copy(deep=True)
    model = meta.load_model(device=device, **kwargs)

    if kwargs:
        logger.info(
            f"Model '{model_name}' loaded with additional arguments: {list(kwargs.keys())}"
        )
        meta.loader_kwargs |= kwargs

    model.mteb_model_meta = meta  # type: ignore[misc]
    return model


_MODEL_RENAMES: dict[str, str] = {
    "bm25s": "mteb/baseline-bm25s",
    # to store model's eval results to display on benchmark
    "baseline/bm25s": "mteb/baseline-bm25s",
    "baseline/random-cross-encoder-baseline": "mteb/baseline-random-cross-encoder",
    "mteb/baseline-random-encoder": "mteb/baseline-random-encoder",
    "baseline/bb25": "mteb/baseline-bb25",
}


def get_model_meta(
    model_name: str,
    revision: str | None = None,
    fetch_from_hf: bool = True,
    fill_missing: bool = False,
    experiment_kwargs: Mapping[str, Any] | None = None,
) -> ModelMeta:
    """A function to fetch a model metadata object by name.

    Args:
        model_name: Name of the model to fetch
        revision: Revision of the model to fetch
        fetch_from_hf: Whether to fetch the model from HuggingFace Hub if not found in the registry
        fill_missing: Fill missing attributes from the metadata including number of parameters and memory usage.
        experiment_kwargs: Optional dictionary of parameters to fill in the metadata for experimental models.

    Returns:
        A model metadata object
    """
    if model_name in _MODEL_RENAMES:
        new_name = _MODEL_RENAMES[model_name]
        msg = f"The model '{model_name}' has been renamed to '{new_name}'. To prevent this warning use the new name."
        warnings.warn(msg, DeprecationWarning, stacklevel=2)
        model_name = new_name

    if model_name in MODEL_REGISTRY:
        model_meta = MODEL_REGISTRY[model_name]

        if revision and (not model_meta.revision == revision):
            raise ValueError(
                f"Model revision {revision} not found for model {model_name}. Expected {model_meta.revision}."
            )

        if experiment_kwargs is not None:
            model_meta = model_meta.model_copy(
                update={"experiment_kwargs": experiment_kwargs}
            )

        if fill_missing and fetch_from_hf:
            original_meta_dict = model_meta.model_dump()
            new_meta = ModelMeta.from_hub(model_name, fill_missing=fill_missing)
            new_meta_dict = new_meta.model_dump(exclude_none=True)

            updates = {
                k: v
                for k, v in new_meta_dict.items()
                if original_meta_dict.get(k) is None
            }

            if updates:
                return model_meta.model_copy(update=updates)
        return model_meta

    if fetch_from_hf:
        logger.info(
            f"Model not found in model registry. Attempting to extract metadata by loading the model ({model_name}) using HuggingFace."
        )
        meta = ModelMeta.from_hub(model_name, revision)
        return meta

    not_found_msg = f"Model '{model_name}' not found in MTEB registry"
    not_found_msg += " nor on the Huggingface Hub." if fetch_from_hf else "."

    close_matches = difflib.get_close_matches(model_name, MODEL_REGISTRY.keys())
    model_names_no_org = {mdl: mdl.split("/")[-1] for mdl in MODEL_REGISTRY.keys()}
    if model_name in model_names_no_org:
        close_matches = [model_names_no_org[model_name]] + close_matches

    suggestion = ""
    if close_matches:
        if len(close_matches) > 1:
            suggestion = f" Did you mean: '{close_matches[0]}' or {close_matches[1]}?"
        else:
            suggestion = f" Did you mean: '{close_matches[0]}'?"

    raise KeyError(not_found_msg + suggestion)



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class STS12STS(AbsTaskSTS):
    min_score = 0
    max_score = 5

    metadata = TaskMetadata(
        name="STS12",
        dataset={
            "path": "mteb/sts12-sts",
            "revision": "a0d554a64d88156834ff5ae9920b964011b16384",
        },
        description="SemEval-2012 Task 6.",
        reference="https://www.aclweb.org/anthology/S12-1051.pdf",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="cosine_spearman",
        date=("2005-01-01", "2012-12-31"),
        domains=["Encyclopaedic", "News", "Written"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{10.5555/2387636.2387697,
  address = {USA},
  author = {Agirre, Eneko and Diab, Mona and Cer, Daniel and Gonzalez-Agirre, Aitor},
  booktitle = {Proceedings of the First Joint Conference on Lexical and Computational Semantics - Volume 1: Proceedings of the Main Conference and the Shared Task, and Volume 2: Proceedings of the Sixth International Workshop on Semantic Evaluation},
  location = {Montr\'{e}al, Canada},
  numpages = {9},
  pages = {385–393},
  publisher = {Association for Computational Linguistics},
  series = {SemEval '12},
  title = {SemEval-2012 task 6: a pilot on semantic textual similarity},
  year = {2012},
}
""",
    )



import random

from datasets import load_dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

TEST_SAMPLES = 2048


class VieQuADRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="VieQuADRetrieval",
        description="A Vietnamese dataset for evaluating Machine Reading Comprehension from Wikipedia articles.",
        reference="https://aclanthology.org/2020.coling-main.233.pdf",
        dataset={
            "path": "taidng/UIT-ViQuAD2.0",
            "revision": "406f09a45cc106a8f7b7fd0c25078883fe58cb1f",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["validation"],
        eval_langs=["vie-Latn"],
        main_score="ndcg_at_10",
        date=("2022-03-02", "2022-03-02"),
        domains=["Encyclopaedic", "Non-fiction", "Written"],
        task_subtypes=["Question answering"],
        license="mit",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{nguyen-etal-2020-vietnamese,
  address = {Barcelona, Spain (Online)},
  author = {Nguyen, Kiet  and
Nguyen, Vu  and
Nguyen, Anh  and
Nguyen, Ngan},
  booktitle = {Proceedings of the 28th International Conference on Computational Linguistics},
  doi = {10.18653/v1/2020.coling-main.233},
  editor = {Scott, Donia  and
Bel, Nuria  and
Zong, Chengqing},
  month = dec,
  pages = {2595--2605},
  publisher = {International Committee on Computational Linguistics},
  title = {A Vietnamese Dataset for Evaluating Machine Reading Comprehension},
  url = {https://aclanthology.org/2020.coling-main.233},
  year = {2020},
}
""",
    )

    def load_data(self, num_proc: int | None = None, **kwargs) -> None:
        if self.data_loaded:
            return

        seed = 42
        random.seed(seed)
        split = self.metadata.eval_splits[0]
        ds = load_dataset(**self.metadata.dataset, split=split)
        ds = ds.shuffle(seed=seed)

        titles, questions, contexts, answers = [], [], [], []
        for row in ds:
            answer = row["answers"]["text"]
            if not answer:
                continue
            titles.append(row["title"])
            questions.append(row["question"])
            contexts.append(row["context"])
            answers.append(answer[0])

        # Downsample after filtering
        max_samples = min(TEST_SAMPLES, len(contexts))
        indices = list(range(len(contexts)))
        random.shuffle(indices)
        indices = indices[:max_samples]
        titles = [titles[idx] for idx in indices]
        questions = [questions[idx] for idx in indices]
        contexts = [contexts[idx] for idx in indices]
        answers = [answers[idx] for idx in indices]

        self.corpus = {split: {}}
        self.relevant_docs = {split: {}}
        self.queries = {split: {}}

        text2id = {}
        n = 0
        for t, q, cont, ans in zip(titles, questions, contexts, answers):
            self.queries[split][str(n)] = q
            q_n = n
            n += 1
            if cont not in text2id:
                text2id[cont] = n
                self.corpus[split][str(n)] = {"title": t, "text": cont}
                n += 1
            if ans not in text2id and ans:
                text2id[ans] = n
                self.corpus[split][str(n)] = {"title": t, "text": ans}
                n += 1

            self.relevant_docs[split][str(q_n)] = {
                str(text2id[ans]): 1,
                str(text2id[cont]): 1,
            }  # only two correct matches

        self.data_loaded = True



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class ReMuQIT2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="ReMuQIT2TRetrieval",
        description="Retrieval of a Wiki passage to answer a query about an image.",
        reference="https://github.com/luomancs/ReMuQ",
        dataset={
            "path": "izhx/UMRB-ReMuQ",
            "revision": "f0bd5955d2897bd1bed56546e88082d966c90a80",
        },
        type="Any2AnyRetrieval",
        category="it2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_5",
        date=("2023-05-15", "2023-07-09"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="cc0-1.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["image", "text"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{luo-etal-2023-end,
  address = {Toronto, Canada},
  author = {Luo, Man  and
Fang, Zhiyuan  and
Gokhale, Tejas  and
Yang, Yezhou  and
Baral, Chitta},
  booktitle = {Proceedings of the 61st Annual Meeting of the Association for Computational Linguistics (Volume 1: Long Papers)},
  doi = {10.18653/v1/2023.acl-long.478},
  editor = {Rogers, Anna  and
Boyd-Graber, Jordan  and
Okazaki, Naoaki},
  month = jul,
  pages = {8573--8589},
  publisher = {Association for Computational Linguistics},
  title = {End-to-end Knowledge Retrieval with Multi-modal Queries},
  url = {https://aclanthology.org/2023.acl-long.478},
  year = {2023},
}
""",
        prompt={
            "query": "Retrieve a fact-based paragraph that provides an answer to the given query about the image."
        },
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_nf_corpus_metadata = dict(
    dataset={
        "path": "clips/beir-nl-nfcorpus",
        "revision": "942953e674fd0f619ff89897abb806dc3df5dd39",
    },
    reference="https://huggingface.co/datasets/clips/beir-nl-nfcorpus",
    type="Retrieval",
    category="t2t",
    modalities=["text"],
    eval_splits=["test"],
    eval_langs=["nld-Latn"],
    main_score="ndcg_at_10",
    date=("2016-03-01", "2016-03-01"),  # best guess: based on publication date
    domains=["Medical", "Academic", "Written"],
    task_subtypes=[],
    license="cc-by-4.0",
    annotations_creators="derived",
    dialect=[],
    sample_creation="machine-translated and verified",  # manually checked a small subset
    bibtex_citation=r"""
@misc{banar2024beirnlzeroshotinformationretrieval,
  archiveprefix = {arXiv},
  author = {Nikolay Banar and Ehsan Lotfi and Walter Daelemans},
  eprint = {2412.08329},
  primaryclass = {cs.CL},
  title = {BEIR-NL: Zero-shot Information Retrieval Benchmark for the Dutch Language},
  url = {https://arxiv.org/abs/2412.08329},
  year = {2024},
}
""",
)


class NFCorpusNL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NFCorpus-NL",
        description="NFCorpus: A Full-Text Learning to Rank Dataset for Medical Information Retrieval. NFCorpus-NL is "
        "a Dutch translation.",
        adapted_from=["NFCorpus"],
        **_nf_corpus_metadata,
    )


class NFCorpusNLv2(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NFCorpus-NL.v2",
        description="NFCorpus: A Full-Text Learning to Rank Dataset for Medical Information Retrieval. NFCorpus-NL is "
        "a Dutch translation. This version adds a Dutch prompt to the dataset.",
        adapted_from=["NFCorpus-NL"],
        prompt={
            "query": "Gegeven een vraag, haal relevante documenten op die de vraag het beste beantwoorden"
        },
        **_nf_corpus_metadata,
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class CSTRVCTKAccentID(AbsTaskClassification):
    metadata = TaskMetadata(
        name="CSTRVCTKAccentID",
        description="Gender classification from CSTR-VCTK dataset. This is a stratified and downsampled version of the original dataset. The dataset was recorded with 2 different microphones, and this mini version uniformly samples data from the 2 microphone types.",
        reference="https://datashare.ed.ac.uk/handle/10283/3443",
        dataset={
            "path": "mteb/cstr-vctk-accent-mini",
            "revision": "ceb854ae5018298bd1b5edb5983b78515ef8ded6",
        },
        type="AudioClassification",
        category="a2t",
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2019-11-13", "2019-11-13"),
        domains=["Spoken", "Speech"],
        task_subtypes=["Accent identification"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Yamagishi2019CSTRVC,
  author = {Junichi Yamagishi and Christophe Veaux and Kirsten MacDonald},
  title = {CSTR VCTK Corpus: English Multi-speaker Corpus for CSTR Voice Cloning Toolkit (version 0.92)},
  url = {https://api.semanticscholar.org/CorpusID:213060286},
  year = {2019},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "accents"



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class RestaurantReviewSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="RestaurantReviewSentimentClassification",
        dataset={
            "path": "hadyelsahar/ar_res_reviews",
            "revision": "d51bf2435d030e0041344f576c5e8d7154828977",
        },
        description="Dataset of 8364 restaurant reviews from qaym.com in Arabic for sentiment analysis",
        reference="https://link.springer.com/chapter/10.1007/978-3-319-18117-2_2",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["ara-Arab"],
        main_score="accuracy",
        date=("2014-01-01", "2015-01-01"),
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=["ara-arab-EG", "ara-arab-JO", "ara-arab-SA"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{elsahar2015building,
  author = {ElSahar, Hady and El-Beltagy, Samhaa R},
  booktitle = {International conference on intelligent text processing and computational linguistics},
  organization = {Springer},
  pages = {23--34},
  title = {Building large arabic multi-domain resources for sentiment analysis},
  year = {2015},
}
""",
        superseded_by="RestaurantReviewSentimentClassification.v2",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        # labels: 0 negative, 1 positive
        self.dataset = self.dataset.rename_column("polarity", "label")
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["train"]
        )


class RestaurantReviewSentimentClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="RestaurantReviewSentimentClassification.v2",
        dataset={
            "path": "mteb/restaurant_review_sentiment",
            "revision": "5d28c1e8fb393173a849696ed178b90a6f78754a",
        },
        description="Dataset of 8156 restaurant reviews from qaym.com in Arabic for sentiment analysis This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2632)",
        reference="https://link.springer.com/chapter/10.1007/978-3-319-18117-2_2",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ara-Arab"],
        main_score="accuracy",
        date=("2014-01-01", "2015-01-01"),
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=["ara-arab-EG", "ara-arab-JO", "ara-arab-SA"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{elsahar2015building,
  author = {ElSahar, Hady and El-Beltagy, Samhaa R},
  booktitle = {International conference on intelligent text processing and computational linguistics},
  organization = {Springer},
  pages = {23--34},
  title = {Building large arabic multi-domain resources for sentiment analysis},
  year = {2015},
}
""",
    )



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class SardiStanceClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SardiStanceClassification",
        dataset={
            "path": "MattiaSangermano/SardiStance",
            "revision": "e25d91e6f6a28ebef42212128f0d5e275b676233",
        },
        description="SardiStance is a unique dataset designed for the task of stance detection in Italian tweets. It consists of tweets related to the Sardines movement, providing a valuable resource for researchers and practitioners in the field of NLP.",
        reference="https://github.com/mirkolai/evalita-sardistance",
        type="Classification",
        category="t2c",
        date=("2019-11-01", "2020-01-31"),
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ita-Latn"],
        main_score="accuracy",
        domains=["Social"],
        task_subtypes=["Political classification"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{cignarella2020sardistance,
  author = {Cignarella, Alessandra Teresa and Lai, Mirko and Bosco, Cristina and Patti, Viviana and Rosso, Paolo and others},
  booktitle = {CEUR WORKSHOP PROCEEDINGS},
  organization = {Ceur},
  pages = {1--10},
  title = {Sardistance@ evalita2020: Overview of the task on stance detection in italian tweets},
  year = {2020},
}
""",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        unused_cols = [
            col
            for col in self.dataset["test"].column_names
            if col not in ["text", "label"]
        ]
        self.dataset = self.dataset.remove_columns(unused_cols)



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class AmazonPolarityClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="AmazonPolarityClassification",
        description="Amazon Polarity Classification Dataset.",
        reference="https://huggingface.co/datasets/amazon_polarity",
        dataset={
            "path": "mteb/amazon_polarity",
            "revision": "e2d317d38cd51312af73b3d32a06d1a08b442046",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2012-01-01",
            "2015-12-31",
        ),  # Estimated range for the collection of reviews
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        annotations_creators="derived",
        license="apache-2.0",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{McAuley2013HiddenFA,
  author = {Julian McAuley and Jure Leskovec},
  journal = {Proceedings of the 7th ACM conference on Recommender systems},
  title = {Hidden factors and hidden topics: understanding rating dimensions with review text},
  url = {https://api.semanticscholar.org/CorpusID:6440341},
  year = {2013},
}
""",
        prompt="Classify Amazon reviews into positive or negative sentiment",
        superseded_by="AmazonPolarityClassification.v2",
    )


class AmazonPolarityClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="AmazonPolarityClassification.v2",
        description="Amazon Polarity Classification Dataset. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://huggingface.co/datasets/amazon_polarity",
        dataset={
            "path": "mteb/amazon_polarity",
            "revision": "ec149c1fe36043668a50804214d4597804001f6f",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2012-01-01",
            "2015-12-31",
        ),  # Estimated range for the collection of reviews
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        annotations_creators="derived",
        license="apache-2.0",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{McAuley2013HiddenFA,
  author = {Julian McAuley and Jure Leskovec},
  journal = {Proceedings of the 7th ACM conference on Recommender systems},
  title = {Hidden factors and hidden topics: understanding rating dimensions with review text},
  url = {https://api.semanticscholar.org/CorpusID:6440341},
  year = {2013},
}
""",
        prompt="Classify Amazon reviews into positive or negative sentiment",
        adapted_from=["AmazonPolarityClassification"],
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class MACSA2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MACSA2TRetrieval",
        description="Audio captions and tags for urban acoustic scenes in TAU Urban Acoustic Scenes 2019 development dataset.",
        reference="https://zenodo.org/records/5114771",
        dataset={
            "path": "mteb/MACS_a2t",
            "revision": "4b34d10b24e99551890d407f12732a2c2ee1405b",
        },
        type="Any2AnyRetrieval",
        category="a2t",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_5",
        date=("2021-01-01", "2021-12-31"),
        domains=["AudioScene"],
        task_subtypes=["Environment Sound Retrieval"],
        license="https://zenodo.org/records/5114771",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{martinmorato2021groundtruthreliabilitymultiannotator,
  archiveprefix = {arXiv},
  author = {Irene Martin-Morato and Annamaria Mesaros},
  eprint = {2104.04214},
  primaryclass = {eess.AS},
  title = {What is the ground truth? Reliability of multi-annotator data for audio tagging},
  url = {https://arxiv.org/abs/2104.04214},
  year = {2021},
}
""",
    )


class MACST2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MACST2ARetrieval",
        description="Audio captions and tags for urban acoustic scenes in TAU Urban Acoustic Scenes 2019 development dataset.",
        reference="https://zenodo.org/records/5114771",
        dataset={
            "path": "mteb/MACS_t2a",
            "revision": "74ce553cb9b43aee5169f0bddd30bd5dee8d26d1",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_5",
        date=("2021-01-01", "2021-12-31"),
        domains=["AudioScene"],
        task_subtypes=["Environment Sound Retrieval"],
        license="https://zenodo.org/records/5114771",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{martinmorato2021groundtruthreliabilitymultiannotator,
  archiveprefix = {arXiv},
  author = {Irene Martin-Morato and Annamaria Mesaros},
  eprint = {2104.04214},
  primaryclass = {eess.AS},
  title = {What is the ground truth? Reliability of multi-annotator data for audio tagging},
  url = {https://arxiv.org/abs/2104.04214},
  year = {2021},
}
""",
    )



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata

N_SAMPLES = 1000


class FinParaSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="FinParaSTS",
        dataset={
            "path": "mteb/FinParaSTS",
            "revision": "16d3834e2eb7f9faedbae0cf25df4b0962c97e71",
        },
        description="Finnish paraphrase-based semantic similarity corpus",
        reference="https://huggingface.co/datasets/TurkuNLP/turku_paraphrase_corpus",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["validation", "test"],
        eval_langs=["fin-Latn"],
        main_score="cosine_spearman",
        date=("2017-01-01", "2021-12-31"),
        domains=["News", "Subtitles", "Written"],
        task_subtypes=[],
        license="cc-by-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{kanerva-etal-2021-finnish,
  address = {Reykjavik, Iceland (Online)},
  author = {Kanerva, Jenna  and
Ginter, Filip  and
Chang, Li-Hsin  and
Rastas, Iiro  and
Skantsi, Valtteri  and
Kilpel{\"a}inen, Jemina  and
Kupari, Hanna-Mari  and
Saarni, Jenna  and
Sev{\'o}n, Maija  and
Tarkka, Otto},
  booktitle = {Proceedings of the 23rd Nordic Conference on Computational Linguistics (NoDaLiDa)},
  editor = {Dobnik, Simon  and
{\\O}vrelid, Lilja},
  month = may # { 31--2 } # jun,
  pages = {288--298},
  publisher = {Link{\"o}ping University Electronic Press, Sweden},
  title = {{F}innish Paraphrase Corpus},
  url = {https://aclanthology.org/2021.nodalida-main.29},
  year = {2021},
}
""",
    )

    min_score = 2
    max_score = 4



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class ChemRxivRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="ChemRxivRetrieval",
        dataset={
            "path": "BASF-AI/ChemRxivRetrieval",
            "revision": "5377aa18f309ec440ff6325a4c2cd3362c2cb8d7",
        },
        description="A retrieval task based on ChemRxiv papers where queries are LLM-synthesized to match specific paragraphs.",
        reference="https://arxiv.org/abs/2508.01643",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2025-01-01", "2025-05-01"),
        domains=["Chemistry"],
        task_subtypes=["Question answering", "Article retrieval"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="LM-generated and reviewed",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@article{kasmaee2025chembed,
  author = {Kasmaee, Ali Shiraee and Khodadad, Mohammad and Astaraki, Mahdi and Saloot, Mohammad Arshi and Sherck, Nicholas and Mahyar, Hamidreza and Samiee, Soheila},
  journal = {arXiv preprint arXiv:2508.01643},
  title = {Chembed: Enhancing chemical literature search through domain-specific text embeddings},
  year = {2025},
}""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class FashionIQIT2IRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="FashionIQIT2IRetrieval",
        description="Retrieve clothes based on descriptions.",
        reference="https://openaccess.thecvf.com/content/CVPR2021/html/Wu_Fashion_IQ_A_New_Dataset_Towards_Retrieving_Images_by_Natural_CVPR_2021_paper.html",
        dataset={
            "path": "mteb/mbeir_fashioniq_task7",
            "revision": "469fba95d895f129f1d297619835d724db01904d",
        },
        type="Any2AnyRetrieval",
        category="it2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2021-01-01", "2021-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="apache-2.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{wu2021fashion,
  author = {Wu, Hui and Gao, Yupeng and Guo, Xiaoxiao and Al-Halah, Ziad and Rennie, Steven and Grauman, Kristen and Feris, Rogerio},
  booktitle = {Proceedings of the IEEE/CVF Conference on computer vision and pattern recognition},
  pages = {11307--11317},
  title = {Fashion iq: A new dataset towards retrieving images by natural language feedback},
  year = {2021},
}
""",
        prompt={
            "query": "Find a fashion image that aligns with the reference image and style note."
        },
    )



from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

from .nvidia_models import nvidia_training_datasets

STELLA_CITATION = """@misc{zhang2025jasperstelladistillationsota,
      title={Jasper and Stella: distillation of SOTA embedding models},
      author={Dun Zhang and Jiacheng Li and Ziyang Zeng and Fulong Wang},
      year={2025},
      eprint={2412.19048},
      archivePrefix={arXiv},
      primaryClass={cs.IR},
      url={https://arxiv.org/abs/2412.19048},
}"""

stella_zh_datasets = {
    "BQ",
    "LCQMC",
    "PAWSX",
    "STS-B",
    "DuRetrieval",
    "AFQMC",
    "Cmnli",
    "Ocnli",
}

# Derived from conversation:

# The model information in Chinese is as follows:
# infgrad/stella-base-zh：based on piccolo-base-zh, using supervised data to train, the data is wudao_base_200GB[1]、m3e[2] and simclue[3]
# infgrad/stella-large-zh：based on piccolo-large-zh, using supervised data to train, the data is wudao_base_200GB[1]、m3e[2] and simclue[3]
# infgrad/stella-base-zh-v2：based on infgrad/stella-base-zh, using supervised data to train, the data is wudao_base_200GB[1]、m3e[2] and simclue[3]
# infgrad/stella-large-zh-v2：based on infgrad/stella-large-zh, using supervised data to train, the data is wudao_base_200GB[1]、m3e[2] and simclue[3]
# For infgrad/stella-mrl-large-zh-v3.5-1792d， infgrad/stella-base-zh-v3-1792d, or other models, I forgot their details, what I remember is that they are distilled models, and using skypile[4] and matrix[5].
# Finally, m3e[2] and simclue[3] has a overlap with C-MTEB, specifically：
# BQ
# lcqmc
# paws-x
# dureader_robust
# AFQMC
# STSB
# CMNLI
# OCNLI
# Totally 8 training datasets are also CMTEB testset.
# https://www.scidb.cn/en/detail?dataSetId=c6a3fe684227415a9db8e21bac4a15ab
# https://github.com/wangyuxinwhy/uniem
# https://github.com/CLUEbenchmark/SimCLUE
# https://huggingface.co/datasets/Skywork/SkyPile-150B
# https://huggingface.co/datasets/m-a-p/Matrix

stella_en_400m = ModelMeta(
    # https://huggingface.co/dunzhang/stella_en_400M_v5/discussions/21#671a6205ac1e2416090f2bf4
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        model_kwargs={"dtype": "auto"},
        trust_remote_code=True,
        apply_instruction_to_passages=False,
    ),
    name="NovaSearch/stella_en_400M_v5",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    use_instructions=True,
    revision="1bb50bc7bb726810eac2140e62155b88b0df198f",
    release_date="2024-07-12",
    n_parameters=435_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=1660,
    max_tokens=8192,
    embed_dim=4096,
    license="mit",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "GritLM",
        "safetensors",
        "Transformers",
    ],
    reference="https://huggingface.co/NovaSearch/stella_en_400M_v5",
    training_datasets=nvidia_training_datasets,  # also distilled from gte-qwen (but training data is unknown) #2164
    public_training_code="https://github.com/NovaSearch-Team/RAG-Retrieval/blob/c40f4638b705eb77d88305d2056901ed550f9f4b/rag_retrieval/train/embedding/README.md",
    public_training_data=None,
    citation=STELLA_CITATION,
)

stella_en_1_5b = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        model_kwargs={"dtype": "auto"},
        trust_remote_code=True,
        apply_instruction_to_passages=False,
    ),
    name="NovaSearch/stella_en_1.5B_v5",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    use_instructions=True,
    revision="d03be74b361d4eb24f42a2fe5bd2e29917df4604",
    release_date="2024-07-12",
    n_parameters=1_540_000_000,
    n_embedding_parameters=232_928_256,
    memory_usage_mb=5887,
    max_tokens=131072,
    embed_dim=8960,
    license="mit",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "GritLM",
        "ONNX",
        "safetensors",
        "Transformers",
    ],
    reference="https://huggingface.co/NovaSearch/stella_en_1.5B_v5",
    training_datasets=nvidia_training_datasets,  # also distilled from gte-qwen (but training data is unknown) #2164
    public_training_code="https://github.com/NovaSearch-Team/RAG-Retrieval/blob/c40f4638b705eb77d88305d2056901ed550f9f4b/rag_retrieval/train/embedding/README.md",
    public_training_data=None,
    citation=STELLA_CITATION,
)

stella_large_zh_v3_1792d = ModelMeta(
    loader=sentence_transformers_loader,
    name="dunzhang/stella-large-zh-v3-1792d",
    model_type=["dense"],
    languages=["zho-Hans"],
    open_weights=True,
    revision="d5d39eb8cd11c80a63df53314e59997074469f09",
    release_date="2024-02-17",
    n_parameters=327359232,
    n_embedding_parameters=21_635_072,
    memory_usage_mb=None,  # can't see on model card
    embed_dim=1792,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/dunzhang/stella-large-zh-v3-1792d",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by="dunzhang/stella-mrl-large-zh-v3.5-1792d",
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,
    training_datasets=stella_zh_datasets,
    # Not in MTEB:
    # - infgrad/dialogue_rewrite_llm
    # - infgrad/retrieval_data_llm
)

stella_base_zh_v3_1792d = ModelMeta(
    loader=sentence_transformers_loader,
    name="infgrad/stella-base-zh-v3-1792d",
    model_type=["dense"],
    languages=["zho-Hans"],
    open_weights=True,
    revision="82254892a0fba125aa2abf3a4800d2dd12821343",
    release_date="2024-02-17",
    n_parameters=104038912,
    n_embedding_parameters=16_226_304,
    memory_usage_mb=None,  # can't see on model card
    embed_dim=1792,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/infgrad/stella-base-zh-v3-1792d",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,
    training_datasets=stella_zh_datasets,
    # Not in MTEB:
    # - infgrad/dialogue_rewrite_llm
    # - infgrad/retrieval_data_llm
)


stella_mrl_large_zh_v3_5_1792d = ModelMeta(
    loader=sentence_transformers_loader,
    name="dunzhang/stella-mrl-large-zh-v3.5-1792d",
    model_type=["dense"],
    languages=["zho-Hans"],
    open_weights=True,
    revision="17bb1c32a93a8fc5f6fc9e91d5ea86da99983cfe",
    release_date="2024-02-27",
    n_parameters=int(326 * 1e6),
    n_embedding_parameters=21_635_072,
    memory_usage_mb=1242,
    embed_dim=1792,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/dunzhang/stella-large-zh-v3-1792d",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    superseded_by=None,
    adapted_from="dunzhang/stella-large-zh-v3-1792d",
    public_training_code=None,
    public_training_data=None,
    training_datasets=stella_zh_datasets,
)

zpoint_large_embedding_zh = ModelMeta(
    loader=sentence_transformers_loader,
    name="iampanda/zpoint_large_embedding_zh",
    model_type=["dense"],
    languages=["zho-Hans"],
    open_weights=True,
    revision="b1075144f440ab4409c05622c1179130ebd57d03",
    release_date="2024-06-04",
    n_parameters=int(326 * 1e6),
    n_embedding_parameters=21_635_072,
    memory_usage_mb=1242,
    embed_dim=1792,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/iampanda/zpoint_large_embedding_zh",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    superseded_by=None,
    adapted_from="dunzhang/stella-mrl-large-zh-v3.5-1792d",
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        # It's a bit unclear what they have trained on to be honest, because they don't list all
        # And they also have some rather cryptic description of their training procedure, but at
        # Least they disclose that they have trained on these:
        "MIRACLRetrieval",
        "MIRACLReranking",
        "DuRetrieval",
        "T2Retrieval",
        "MultiLongDocRetrieval",
        #  Not in MTEB:
        #  - Shitao/bge-reranker-data
        #  - FreedomIntelligence/Huatuo26M-Lite
    },
)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

if TYPE_CHECKING:
    from mteb.types import PromptType
logger = logging.getLogger(__name__)


def instruction_template(
    instruction: str, prompt_type: PromptType | None = None
) -> str:
    return f"Instruct: {instruction}\nQuery: " if instruction else ""


hinvec_training_datasets = {
    "MintakaRetrieval",
    "HindiDiscourseClassification",
    "SentimentAnalysisHindi",
    "MassiveScenarioClassification",
    "MTOPIntentClassification",
    "LinceMTBitextMining",
    "PhincBitextMining",
    "XNLI",
    "MLQARetrieval",
    "FloresBitextMining",
    "AmazonReviewsClassification",
}

Hinvec_bidir = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        instruction_template=instruction_template,
        trust_remote_code=True,
        max_seq_length=2048,
        padding_side="left",
        add_eos_token=True,
    ),
    name="Sailesh97/Hinvec",
    model_type=["dense"],
    languages=["eng-Latn", "hin-Deva"],
    open_weights=True,
    revision="d4fc678720cc1b8c5d18599ce2d9a4d6090c8b6b",
    release_date="2025-06-19",
    n_parameters=939_591_680,
    n_embedding_parameters=67108864,
    memory_usage_mb=3715,
    embed_dim=2048,
    license="cc-by-nc-4.0",
    max_tokens=2048,
    reference="https://huggingface.co/Sailesh97/Hinvec",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=True,
    training_datasets=hinvec_training_datasets,
    public_training_code=None,
    public_training_data=None,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class FaithDialRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="FaithDial",
        dataset={
            "path": "mteb/FaithDial",
            "revision": "303f0593746993a53c0c2ec976fb576369f6c9f3",
        },
        reference="https://mcgill-nlp.github.io/FaithDial",
        description=(
            "FaithDial is a faithful knowledge-grounded dialogue benchmark."
            + "It was curated by asking annotators to amend hallucinated utterances in Wizard of Wikipedia (WoW). "
            + "It consists of conversation histories along with manually labelled relevant passage. "
            + "For the purpose of retrieval, we only consider the instances marked as 'Edification' in the VRM field, "
            + "as the gold passage associated with these instances is non-ambiguous."
        ),
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2022-01-01", "2022-03-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Conversational retrieval"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{dziri2022faithdial,
  author = {Dziri, Nouha and Kamalloo, Ehsan and Milton, Sivan and Zaiane, Osmar and Yu, Mo and Ponti, Edoardo M and Reddy, Siva},
  doi = {10.1162/tacl_a_00529},
  journal = {Transactions of the Association for Computational Linguistics},
  month = {12},
  pages = {1473--1490},
  publisher = {MIT Press},
  title = {{FaithDial: A Faithful Benchmark for Information-Seeking Dialogue}},
  volume = {10},
  year = {2022},
}
""",
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm
from transformers import SEWDForCTC, Wav2Vec2FeatureExtractor

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_audio_dependencies
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput


class SewDWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_seconds: float = 30.0,
        **kwargs: Any,
    ):
        requires_audio_dependencies()
        self.model_name = model_name
        self.device = device
        self.max_audio_length_seconds = max_audio_length_seconds

        # SewD uses the same feature extractor as Wav2Vec2
        self.feature_extractor = Wav2Vec2FeatureExtractor.from_pretrained(
            model_name, revision=revision
        )
        self.model = SEWDForCTC.from_pretrained(model_name, revision=revision).to(
            self.device
        )
        self.model.eval()
        self.sampling_rate = self.feature_extractor.sampling_rate

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        inputs.collate_fn = AudioCollator(self.sampling_rate)
        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_arrays = [audio["array"] for audio in batch["audio"]]
            feature_inputs = self.feature_extractor(
                audio_arrays,
                sampling_rate=self.sampling_rate,
                return_tensors="pt",
                padding="longest",
                truncation=True,
                max_length=int(self.max_audio_length_seconds * self.sampling_rate),
                return_attention_mask=True,
            ).to(self.device)

            with torch.no_grad():
                # SewD model outputs
                outputs = self.model(
                    feature_inputs.input_values,
                    attention_mask=feature_inputs.attention_mask,
                    output_hidden_states=True,
                )

                # Get embeddings from last hidden state
                last_hidden_state = outputs.hidden_states[-1]

                # Apply attention-masked pooling to exclude padding tokens
                batch_size, hidden_seq_len, hidden_size = last_hidden_state.shape
                device = last_hidden_state.device

                # Calculate proper hidden lengths based on input attention mask
                input_lengths = feature_inputs.attention_mask.sum(dim=1)
                downsample_ratio = feature_inputs.input_values.shape[1] / hidden_seq_len
                hidden_lengths = (input_lengths.float() / downsample_ratio).long()
                hidden_lengths = torch.clamp(hidden_lengths, min=0, max=hidden_seq_len)

                # Create attention mask for hidden states
                seq_range = torch.arange(hidden_seq_len, device=device).unsqueeze(0)
                hidden_attention_mask = (seq_range < hidden_lengths.unsqueeze(1)).to(
                    last_hidden_state.dtype
                )

                # Apply masked mean pooling
                hidden_attention_mask = hidden_attention_mask.unsqueeze(-1)
                masked_embeddings = last_hidden_state * hidden_attention_mask
                valid_tokens = hidden_attention_mask.sum(dim=1)
                embeddings = masked_embeddings.sum(dim=1) / valid_tokens.clamp(min=1e-9)

                all_embeddings.append(embeddings.cpu().detach())

        return torch.cat(all_embeddings, dim=0).numpy()

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "audio" not in inputs.dataset.features:
            raise ValueError("SewDWrapper only supports audio inputs.")
        return self.get_audio_embeddings(inputs, **kwargs)


sewd_base = ModelMeta(
    loader=SewDWrapper,
    name="asapp/sew-d-base-plus-400k-ft-ls100h",
    languages=["eng-Latn"],
    open_weights=True,
    revision="d78e7a1b50e9f1ce21887ca069330efdd5ccd4ca",
    release_date="2021-09-14",
    max_tokens=float("inf"),
    n_parameters=95_000_000,
    memory_usage_mb=675,
    embed_dim=768,
    license="apache-2.0",
    reference="https://huggingface.co/asapp/sew-d-base-plus-400k-ft-ls100h",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={"LibriSpeech"},
    modalities=["audio"],
    citation="""
@misc{wu2021performanceefficiencytradeoffsunsupervisedpretraining,
      title={Performance-Efficiency Trade-offs in Unsupervised Pre-training for Speech Recognition},
      author={Felix Wu and Kwangyoun Kim and Jing Pan and Kyu Han and Kilian Q. Weinberger and Yoav Artzi},
      year={2021},
      eprint={2109.06870},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2109.06870},
}""",
)

sewd_tiny = ModelMeta(
    loader=SewDWrapper,
    name="asapp/sew-d-tiny-100k-ft-ls100h",
    languages=["eng-Latn"],
    open_weights=True,
    revision="1966cdcfbd2123ee90b003c2aa6ec6fe204cc4d8",
    release_date="2021-09-14",
    max_tokens=float("inf"),
    n_parameters=19_700_000,
    memory_usage_mb=92,
    embed_dim=256,
    license="apache-2.0",
    reference="https://huggingface.co/asapp/sew-d-tiny-100k-ft-ls100h",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={"LibriSpeech"},
    modalities=["audio"],
    citation="""
@misc{wu2021performanceefficiencytradeoffsunsupervisedpretraining,
      title={Performance-Efficiency Trade-offs in Unsupervised Pre-training for Speech Recognition},
      author={Felix Wu and Kwangyoun Kim and Jing Pan and Kyu Han and Kilian Q. Weinberger and Yoav Artzi},
      year={2021},
      eprint={2109.06870},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2109.06870},
}""",
)

sewd_mid = ModelMeta(
    loader=SewDWrapper,
    name="asapp/sew-d-mid-400k-ft-ls100h",
    languages=["eng-Latn"],
    open_weights=True,
    revision="b2ff9fdb3bddc81657cf5f16bc0c510be0a39b3e",
    release_date="2021-09-14",
    max_tokens=float("inf"),
    n_parameters=139_000_000,
    memory_usage_mb=530,
    embed_dim=768,
    license="apache-2.0",
    reference="https://huggingface.co/asapp/sew-d-mid-400k-ft-ls100h",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={"LibriSpeech"},
    modalities=["audio"],
    citation="""
@misc{wu2021performanceefficiencytradeoffsunsupervisedpretraining,
      title={Performance-Efficiency Trade-offs in Unsupervised Pre-training for Speech Recognition},
      author={Felix Wu and Kwangyoun Kim and Jing Pan and Kyu Han and Kilian Q. Weinberger and Yoav Artzi},
      year={2021},
      eprint={2109.06870},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2109.06870},
}""",
)



from mteb.abstasks import AbsTaskClustering
from mteb.abstasks.task_metadata import TaskMetadata


class VoxCelebClustering(AbsTaskClustering):
    label_column_name: str = "label_id"
    metadata = TaskMetadata(
        name="VoxCelebClustering",
        description="Clustering task based on the VoxCeleb dataset for sentiment analysis, clustering by positive/negative sentiment.",
        reference="https://huggingface.co/datasets/DynamicSuperb/Sentiment_Analysis_SLUE-VoxCeleb",
        dataset={
            "path": "mteb/Sentiment_Analysis_SLUE-VoxCeleb",
            "revision": "3155df63aedc31a112c284097fdd7c0316a24ee8",
        },
        type="AudioClustering",
        category="a2a",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2024-06-27", "2024-06-28"),
        domains=["Spoken", "Speech"],
        task_subtypes=["Sentiment Clustering"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@misc{shon2022sluenewbenchmarktasks,
  archiveprefix = {arXiv},
  author = {Suwon Shon and Ankita Pasad and Felix Wu and Pablo Brusco and Yoav Artzi and Karen Livescu and Kyu J. Han},
  eprint = {2111.10367},
  primaryclass = {cs.CL},
  title = {SLUE: New Benchmark Tasks for Spoken Language Understanding Evaluation on Natural Speech},
  url = {https://arxiv.org/abs/2111.10367},
  year = {2022},
}
""",
    )
    max_fraction_of_documents_to_embed = None
    input_column_name: str = "audio"

    def dataset_transform(self):
        ds = self.dataset
        # Remove 'Disagreement' samples and '<mixed>' samples
        ds = ds.filter(lambda x: x["label"] not in ["Disagreement", "<mixed>"])
        # Map string sentiment labels to numeric IDs
        label2id = {"Negative": 0, "Neutral": 1, "Positive": 2}

        def add_label_id(example):
            example["label_id"] = label2id[example["label"]]
            return example

        ds = ds.map(add_label_id)
        self.dataset = ds
        self.label_column_name = "label_id"

        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"], label=self.label_column_name
        )



from mteb.models.model_meta import (
    ModelMeta,
    ScoringFunction,
)
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

GRANITE_EMBEDDING_CITATION = """@article{awasthy2025graniteembedding,
  title={Granite Embedding Models},
  author={Awasthy, Parul and Trivedi, Aashka and Li, Yulong and Bornea, Mihaela and Cox, David and Daniels, Abraham and Franz, Martin and Goodhart, Gabe and Iyer, Bhavani and Kumar, Vishwajeet and Lastras, Luis and McCarley, Scott and Murthy, Rudra and P, Vignesh and Rosenthal, Sara and Roukos, Salim and Sen, Jaydeep and Sharma, Sukriti and Sil, Avirup and Soule, Kate and Sultan, Arafat and Florian, Radu},
  journal={arXiv preprint arXiv:2502.20204},
  year={2025}
}"""

GRANITE_LANGUAGES = [
    "ara-Latn",
    "ces-Latn",
    "deu-Latn",
    "eng-Latn",
    "spa-Latn",
    "fra-Latn",
    "ita-Latn",
    "jpn-Latn",
    "kor-Latn",
    "nld-Latn",
    "por-Latn",
    "zho-Hant",
    "zho-Hans",
]

granite_training_data = {
    # Multilingual MC4
    # Multilingual Webhose
    # English Wikipedia
    # Multilingual Wikimedia
    "WikipediaRetrievalMultilingual",
    "WikipediaRerankingMultilingual",
    # Miracl Corpus (Title-Body)
    # Stack Exchange Duplicate questions (titles)
    # Stack Exchange Duplicate questions (titles)
    # Stack Exchange Duplicate questions (bodies)
    "StackOverflowDupQuestions",
    "AskUbuntuDupQuestions",
    # Stack Exchange (Title, Answer) pairs
    # Stack Exchange (Title, Body) pairs
    # Stack Exchange (Title, Body) pairs
    # Machine Translations of Stack Exchange Duplicate questions (titles)
    # Machine Translations of Stack Exchange (Title+Body, Answer) pairs
    "StackExchangeClusteringP2P",
    "StackExchangeClusteringP2P.v2",
    "StackExchangeClustering",
    "StackExchangeClustering.v2",
    # SearchQA
    # S2ORC (Title, Abstract)
    # WikiAnswers Duplicate question pairs
    # CCNews
    # XSum
    # SimpleWiki
    # Machine Translated Cross Lingual Parallel Corpora
    # SPECTER citation triplets
    # Machine Translations of SPECTER citation triplets
    # Natural Questions (NQ)
    "NQ",
    "NQ-NL",  # translation not trained on
    "NQHardNegatives",
    # SQuAD2.0
    # HotpotQA
    "HotPotQA",
    "HotPotQAHardNegatives",
    "HotPotQA-PL",  # translated from hotpotQA (not trained on)
    "HotpotQA-NL",  # translated from hotpotQA (not trained on)
    # Fever
    "FEVER",
    "FEVERHardNegatives",
    "FEVER-NL",  # translated from hotpotQA (not trained on)
    # PubMed
    # Multilingual Miracl Triples
    "MIRACLRetrieval",
    "MIRACLRetrievalHardNegatives",
    "MIRACLReranking",
    # Multilingual MrTydi Triples
    "MrTidyRetrieval",
    # Sadeeem Question Answering
    # DBPedia Title-Body Pairs
    "DBPedia",
    "DBPedia-NL",  # translated from hotpotQA (not trained on)
    # Synthetic: English Query-Wikipedia Passage
    # Synthetic: English Fact Verification
    # Synthetic: Multilingual Query-Wikipedia Passage
    # Synthetic: Multilingual News Summaries
    # IBM Internal Triples
    # IBM Internal Title-Body Pairs
}

granite_107m_multilingual = ModelMeta(
    loader=sentence_transformers_loader,
    name="ibm-granite/granite-embedding-107m-multilingual",
    model_type=["dense"],
    languages=GRANITE_LANGUAGES,
    open_weights=True,
    revision="47db56afe692f731540413c67dd818ff492277e7",
    release_date="2024-12-18",
    n_parameters=106994304,
    n_embedding_parameters=96_000_768,
    memory_usage_mb=204,
    embed_dim=384,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/ibm-granite/granite-embedding-107m-multilingual",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "Transformers",
        "ONNX",
        "safetensors",
    ],
    adapted_from=None,
    superseded_by=None,
    public_training_code=None,
    public_training_data=None,
    use_instructions=False,
    training_datasets=granite_training_data,
    citation=GRANITE_EMBEDDING_CITATION,
)

granite_278m_multilingual = ModelMeta(
    loader=sentence_transformers_loader,
    name="ibm-granite/granite-embedding-278m-multilingual",
    model_type=["dense"],
    languages=GRANITE_LANGUAGES,
    open_weights=True,
    revision="84e3546b88b0cb69f8078608a1df558020bcbf1f",
    release_date="2024-12-18",
    n_parameters=278043648,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=530,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/ibm-granite/granite-embedding-278m-multilingual",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "Transformers",
        "ONNX",
        "safetensors",
    ],
    adapted_from=None,
    superseded_by=None,
    public_training_code=None,
    public_training_data=None,
    use_instructions=False,
    training_datasets=granite_training_data,
    citation=GRANITE_EMBEDDING_CITATION,
)

granite_30m_english = ModelMeta(
    loader=sentence_transformers_loader,
    name="ibm-granite/granite-embedding-30m-english",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="eddbb57470f896b5f8e2bfcb823d8f0e2d2024a5",
    release_date="2024-12-18",
    n_parameters=30295296,
    n_embedding_parameters=19_301_760,
    memory_usage_mb=58,
    embed_dim=384,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/ibm-granite/granite-embedding-30m-english",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "ONNX",
        "safetensors",
        "Transformers",
    ],
    adapted_from=None,
    superseded_by=None,
    public_training_code=None,
    public_training_data=None,
    use_instructions=False,
    training_datasets=granite_training_data,
    citation=GRANITE_EMBEDDING_CITATION,
)

granite_125m_english = ModelMeta(
    loader=sentence_transformers_loader,
    name="ibm-granite/granite-embedding-125m-english",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="e48d3a5b47eaa18e3fe07d4676e187fd80f32730",
    release_date="2024-12-18",
    n_parameters=124645632,
    n_embedding_parameters=38_603_520,
    memory_usage_mb=238,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/ibm-granite/granite-embedding-125m-english",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "ONNX",
        "safetensors",
        "Transformers",
    ],
    adapted_from=None,
    superseded_by=None,
    public_training_code=None,
    public_training_data=None,
    use_instructions=False,
    training_datasets=granite_training_data,
    citation=GRANITE_EMBEDDING_CITATION,
)


granite_english_r2 = ModelMeta(
    loader=sentence_transformers_loader,
    name="ibm-granite/granite-embedding-english-r2",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="6e7b8ce0e76270394ac4669ba4bbd7133b60b7f9",
    release_date="2025-08-15",
    n_parameters=149014272,
    n_embedding_parameters=38682624,
    memory_usage_mb=284,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=8192,
    reference="https://huggingface.co/ibm-granite/granite-embedding-english-r2",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    adapted_from=None,
    superseded_by=None,
    public_training_code=None,
    public_training_data=None,
    use_instructions=False,
    training_datasets=granite_training_data,
    citation=GRANITE_EMBEDDING_CITATION,
)

granite_small_english_r2 = ModelMeta(
    loader=sentence_transformers_loader,
    name="ibm-granite/granite-embedding-small-english-r2",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="54a8d2616a0844355a5164432d3f6dafb37b17a3",
    release_date="2025-08-15",
    n_parameters=47662464,
    n_embedding_parameters=19341312,
    memory_usage_mb=91,
    embed_dim=384,
    license="apache-2.0",
    max_tokens=8192,
    reference="https://huggingface.co/ibm-granite/granite-embedding-small-english-r2",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    adapted_from=None,
    superseded_by=None,
    public_training_code=None,
    public_training_data=None,
    use_instructions=False,
    training_datasets=granite_training_data,
    citation=GRANITE_EMBEDDING_CITATION,
)



import importlib.util
import logging

logger = logging.getLogger(__name__)


def _is_package_available(pkg_name: str) -> bool:
    package_exists = importlib.util.find_spec(pkg_name) is not None
    return package_exists


def requires_package(
    obj, package_name: str, model_name: str, install_instruction: str | None = None
) -> None:
    """Check if a package is available and raise an error with installation instructions if it's not.

    Args:
        obj: The object (class or function) that requires the package.
        package_name: The name of the package to check.
        model_name: The name of the model that benefits from the package.
        install_instruction: The instruction to install the package. If None, defaults to "pip install {package_name}".
    """
    if not _is_package_available(package_name):
        install_instruction = (
            f"pip install {package_name}"
            if install_instruction is None
            else install_instruction
        )
        name = obj.__name__ if hasattr(obj, "__name__") else obj.__class__.__name__
        raise ImportError(
            f"{name} requires the `{package_name}` library but it was not found in your environment. "
            + f"If you want to load {model_name} models, please `{install_instruction}` to install the package."
        )


def suggest_package(
    obj: object, package_name: str, model_name: str, install_instruction: str
) -> bool:
    """Suggestion to install a package.

    Check if a package is available and log a warning with installation instructions if it's not.
    Unlike requires_package, this doesn't raise an error but returns True if the package is available.

    Args:
        obj: The object (class or function) that requires the package.
        package_name: The name of the package to check.
        model_name: The name of the model that benefits from the package.
        install_instruction: The instruction to install the package.

    Returns:
        bool: True if the package is available, False otherwise.
    """
    if not _is_package_available(package_name):
        name = obj.__name__ if hasattr(obj, "__name__") else obj.__class__.__name__
        logger.warning(
            f"{name} can benefit from the `{package_name}` library but it was not found in your environment. "
            + f"{model_name} models were trained with flash attention enabled. For optimal performance, please install the `{package_name}` package with `{install_instruction}`."
        )
        return False
    return True


def requires_image_dependencies() -> None:
    """Check if the required dependencies for image tasks are available."""
    if not _is_package_available("torchvision"):
        raise ImportError(
            "You are trying to running the image subset of mteb without having installed the required dependencies (`torchvision`). "
            + "You can install the required dependencies using `pip install 'mteb[image]'` to install the required dependencies."
        )


def requires_audio_dependencies() -> None:
    """Check if the required dependencies for audio tasks are available."""
    if not _is_package_available("torchaudio"):
        raise ImportError(
            "You are trying to running the audio subset of mteb without having installed the required dependencies (`torchaudio`). "
            + "You can install the required dependencies using `pip install 'mteb[audio]'` to install the required dependencies."
        )



"""Event Logger - Event Logger

A lightweight event logging SDK for Gradio projects

Features:
- Respects user privacy, does not record any user information
- Uses MongoDB for storage
- Asynchronous writes, non-blocking main thread
- Silent failure, does not affect main business logic
- Concise API design

Usage example:
    ```python
    import gradio as gr
    from event_logger import EventLogger, get_session_js

    # Initialize (optional: set MONGO_URI to enable event logging)
    logger = EventLogger()

    with gr.Blocks() as demo:
        session_id = gr.State()
        benchmark_select = gr.Dropdown(choices=["MTEB-English", "MTEB-Chinese"])
        type_select = gr.Dropdown(choices=["all", "classification", "clustering"])

        # Initialize session_id
        demo.load(
            fn=lambda x: x,
            inputs=[],
            outputs=[session_id],
            js=get_session_js()
        )

        # Log events in callbacks
        def on_type_change(type_val, benchmark, session_id):
            # Your business logic...

            # Log event
            logger.log_filter_change(
                session_id=session_id,
                filter_name="task_type",
                new_value=type_val,
                benchmark=benchmark
            )
            return type_val

        type_select.change(
            on_type_change,
            inputs=[type_select, benchmark_select, session_id],
            outputs=[type_select]
        )
    ```
"""

from .logger import EventLogger
from .models import (
    BaseEvent,
    BenchmarkChangeEvent,
    FilterChangeEvent,
    PageViewEvent,
    TableDownloadEvent,
    TableSwitchEvent,
)

__all__ = [
    "BaseEvent",
    "BenchmarkChangeEvent",
    "EventLogger",
    "FilterChangeEvent",
    "PageViewEvent",
    "TableDownloadEvent",
    "TableSwitchEvent",
]



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_audio_dependencies, requires_package
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput


class CNN14Wrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_s: float = 30.0,
        **kwargs: Any,
    ):
        requires_audio_dependencies()
        self.model_name = model_name
        self.device = device
        self.max_audio_length_s = max_audio_length_s

        requires_package(
            self,
            "speechbrain",
            "speechbrain/cnn14-esc50",
            "pip install 'mteb[speechbrain]'",
        )

        from speechbrain.inference.classifiers import AudioClassifier

        # Load the SpeechBrain model
        self.model = AudioClassifier.from_hparams(
            source=model_name,
            savedir="pretrained_models/cnn14-esc50",
            run_opts={"device": device},
        )

        # SpeechBrain uses a 16kHz sampling rate for audio
        self.sampling_rate = 16_000

    def _pad_audio_batch(self, batch: list[torch.Tensor]) -> torch.Tensor:
        max_len = max(w.shape[0] for w in batch)
        padded = [torch.nn.functional.pad(w, (0, max_len - w.shape[0])) for w in batch]
        return torch.stack(padded)

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        inputs.collate_fn = AudioCollator(self.sampling_rate)

        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_tensors = []
            for a in batch["audio"]:
                array = torch.tensor(a["array"], dtype=torch.float32)

                array = array.squeeze()

                # Apply audio truncation (configurable limit)
                max_length = int(self.max_audio_length_s * self.sampling_rate)
                if array.shape[-1] > max_length:
                    array = array[..., :max_length]

                audio_tensors.append(array)

            with torch.no_grad():
                # Convert batch to tensors and move to device
                batch_tensor = self._pad_audio_batch(audio_tensors).to(self.device)

                feats = self.model.mods.compute_features(batch_tensor)
                b, f, t = feats.shape
                if f < 64 or t < 80:
                    # zero-pad in the frequency or time dimension until it's at least [64, 80]
                    pad_freq = max(0, 64 - f)
                    pad_time = max(0, 80 - t)
                    feats = torch.nn.functional.pad(feats, (0, pad_time, 0, pad_freq))
                embeddings = self.model.mods.embedding_model(feats)
                # Apply mean pooling over time dimension if needed
                if embeddings.dim() > 2:
                    embeddings = torch.mean(embeddings, dim=1)

                all_embeddings.append(embeddings.cpu().detach())

        return torch.cat(all_embeddings, dim=0).numpy()

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "audio" not in inputs.dataset.features:
            raise ValueError("ASTWrapper only supports audio inputs.")
        return self.get_audio_embeddings(inputs, **kwargs)


cnn14_esc50 = ModelMeta(
    loader=CNN14Wrapper,
    name="speechbrain/cnn14-esc50",
    languages=["eng-Latn"],
    open_weights=True,
    revision="422a112e9a22a5fac0d37571aacaee5caf154395",
    release_date="2022-11-26",
    max_tokens=None,
    n_parameters=80_753_615,
    memory_usage_mb=308,
    embed_dim=2048,
    license="apache-2.0",
    reference="https://huggingface.co/speechbrain/cnn14-esc50",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/speechbrain/speechbrain",
    public_training_data=None,
    training_datasets=None,  # ["ESC-50", "VGGSound"],
    modalities=["audio"],
    citation="""
@inproceedings{wang2022CRL,
    title={Learning Representations for New Sound Classes With Continual Self-Supervised Learning},
    author={Zhepei Wang, Cem Subakan, Xilin Jiang, Junkai Wu, Efthymios Tzinis, Mirco Ravanelli, Paris Smaragdis},
    year={2022},
    booktitle={Accepted to IEEE Signal Processing Letters}
}
""",
)
